@@ -2,7 +2,6 @@
 
 
 
-
 package libsvm;
 import java.io.*;
 import java.util.*;
@@ -81,7 +80,7 @@ class Cache {
 			if(h.data != null) System.arraycopy(h.data,0,new_data,0,h.len);
 			h.data = new_data;
 			size -= more;
-			do {int tmp=h.len; h.len=len; len=tmp;} while(false);
+			do {int _=h.len; h.len=len; len=_;} while(false);
 		}
 
 		lru_insert(h);
@@ -92,21 +91,21 @@ class Cache {
 	void swap_index(int i, int j)
 	{
 		if(i==j) return;
-
+		
 		if(head[i].len > 0) lru_delete(head[i]);
 		if(head[j].len > 0) lru_delete(head[j]);
-		do {float[] tmp=head[i].data; head[i].data=head[j].data; head[j].data=tmp;} while(false);
-		do {int tmp=head[i].len; head[i].len=head[j].len; head[j].len=tmp;} while(false);
+		do {float[] _=head[i].data; head[i].data=head[j].data; head[j].data=_;} while(false);
+		do {int _=head[i].len; head[i].len=head[j].len; head[j].len=_;} while(false);
 		if(head[i].len > 0) lru_insert(head[i]);
 		if(head[j].len > 0) lru_insert(head[j]);
 
-		if(i>j) do {int tmp=i; i=j; j=tmp;} while(false);
+		if(i>j) do {int _=i; i=j; j=_;} while(false);
 		for(head_t h = lru_head.next; h!=lru_head; h=h.next)
 		{
 			if(h.len > i)
 			{
 				if(h.len > j)
-					do {float tmp=h.data[i]; h.data[i]=h.data[j]; h.data[j]=tmp;} while(false);
+					do {float _=h.data[i]; h.data[i]=h.data[j]; h.data[j]=_;} while(false);
 				else
 				{
 					// give up
@@ -148,8 +147,8 @@ abstract class Kernel extends QMatrix {
 
 	void swap_index(int i, int j)
 	{
-		do {svm_node[] tmp=x[i]; x[i]=x[j]; x[j]=tmp;} while(false);
-		if(x_square != null) do {double tmp=x_square[i]; x_square[i]=x_square[j]; x_square[j]=tmp;} while(false);
+		do {svm_node[] _=x[i]; x[i]=x[j]; x[j]=_;} while(false);
+		if(x_square != null) do {double _=x_square[i]; x_square[i]=x_square[j]; x_square[j]=_;} while(false);
 	}
 
 	private static double powi(double base, int times)
@@ -179,7 +178,7 @@ abstract class Kernel extends QMatrix {
 			case svm_parameter.PRECOMPUTED:
 				return x[i][(int)(x[j][0].value)].value;
 			default:
-				return 0;	// Unreachable
+				return 0;	// java
 		}
 	}
 
@@ -198,8 +197,7 @@ abstract class Kernel extends QMatrix {
 			for(int i=0;i<l;i++)
 				x_square[i] = dot(x[i],x[i]);
 		}
-		else
-			x_square = null;
+		else x_square = null;
 	}
 
 	static double dot(svm_node[] x, svm_node[] y)
@@ -275,10 +273,10 @@ abstract class Kernel extends QMatrix {
 			}
 			case svm_parameter.SIGMOID:
 				return Math.tanh(param.gamma*dot(x,y)+param.coef0);
-			case svm_parameter.PRECOMPUTED:  //x: test (validation), y: SV
+			case svm_parameter.PRECOMPUTED:
 				return	x[(int)(y[0].value)].value;
 			default:
-				return 0;	// Unreachable
+				return 0;	// java
 		}
 	}
 }
@@ -319,7 +317,7 @@ class Solver {
 	double[] G_bar;		// gradient, if we treat free variables as 0
 	int l;
 	boolean unshrink;	// XXX
-
+	
 	static final double INF = java.lang.Double.POSITIVE_INFINITY;
 
 	double get_C(int i)
@@ -336,7 +334,7 @@ class Solver {
 	}
 	boolean is_upper_bound(int i) { return alpha_status[i] == UPPER_BOUND; }
 	boolean is_lower_bound(int i) { return alpha_status[i] == LOWER_BOUND; }
-	boolean is_free(int i) { return alpha_status[i] == FREE; }
+	boolean is_free(int i) {  return alpha_status[i] == FREE; }
 
 	// java: information about solution except alpha,
 	// because we cannot return multiple values otherwise...
@@ -351,13 +349,13 @@ class Solver {
 	void swap_index(int i, int j)
 	{
 		Q.swap_index(i,j);
-		do {byte tmp=y[i]; y[i]=y[j]; y[j]=tmp;} while(false);
-		do {double tmp=G[i]; G[i]=G[j]; G[j]=tmp;} while(false);
-		do {byte tmp=alpha_status[i]; alpha_status[i]=alpha_status[j]; alpha_status[j]=tmp;} while(false);
-		do {double tmp=alpha[i]; alpha[i]=alpha[j]; alpha[j]=tmp;} while(false);
-		do {double tmp=p[i]; p[i]=p[j]; p[j]=tmp;} while(false);
-		do {int tmp=active_set[i]; active_set[i]=active_set[j]; active_set[j]=tmp;} while(false);
-		do {double tmp=G_bar[i]; G_bar[i]=G_bar[j]; G_bar[j]=tmp;} while(false);
+		do {byte _=y[i]; y[i]=y[j]; y[j]=_;} while(false);
+		do {double _=G[i]; G[i]=G[j]; G[j]=_;} while(false);
+		do {byte _=alpha_status[i]; alpha_status[i]=alpha_status[j]; alpha_status[j]=_;} while(false);
+		do {double _=alpha[i]; alpha[i]=alpha[j]; alpha[j]=_;} while(false);
+		do {double _=p[i]; p[i]=p[j]; p[j]=_;} while(false);
+		do {int _=active_set[i]; active_set[i]=active_set[j]; active_set[j]=_;} while(false);
+		do {double _=G_bar[i]; G_bar[i]=G_bar[j]; G_bar[j]=_;} while(false);
 	}
 
 	void reconstruct_gradient()
@@ -387,7 +385,7 @@ class Solver {
 				for(j=0;j<active_size;j++)
 					if(is_free(j))
 						G[i] += alpha[j] * Q_i[j];
-			}
+			}	
 		}
 		else
 		{
@@ -485,7 +483,7 @@ class Solver {
 				else
 					counter = 1;	// do shrinking next iteration
 			}
-
+			
 			int i = working_set[0];
 			int j = working_set[1];
 
@@ -511,7 +509,7 @@ class Solver {
 				double diff = alpha[i] - alpha[j];
 				alpha[i] += delta;
 				alpha[j] += delta;
-
+			
 				if(diff > 0)
 				{
 					if(alpha[j] < 0)
@@ -631,7 +629,7 @@ class Solver {
 			}
 
 		}
-
+		
 		if(iter >= max_iter)
 		{
 			if(active_size < l)
@@ -641,7 +639,7 @@ class Solver {
 				active_size = l;
 				svm.info("*");
 			}
-			System.err.print("\nWARNING: reaching max number of iterations\n");
+			svm.info("\nWARNING: reaching max number of iterations");
 		}
 
 		// calculate rho
@@ -675,18 +673,18 @@ class Solver {
 	{
 		// return i,j such that
 		// i: maximizes -y_i * grad(f)_i, i in I_up(\alpha)
-		// j: minimizes the decrease of obj value
+		// j: mimimizes the decrease of obj value
 		//    (if quadratic coefficeint <= 0, replace it with tau)
 		//    -y_j*grad(f)_j < -y_i*grad(f)_i, j in I_low(\alpha)
-
+		
 		double Gmax = -INF;
 		double Gmax2 = -INF;
 		int Gmax_idx = -1;
 		int Gmin_idx = -1;
 		double obj_diff_min = INF;
-
+	
 		for(int t=0;t<active_size;t++)
-			if(y[t]==+1)
+			if(y[t]==+1)	
 			{
 				if(!is_upper_bound(t))
 					if(-G[t] >= Gmax)
@@ -704,12 +702,12 @@ class Solver {
 						Gmax_idx = t;
 					}
 			}
-
+	
 		int i = Gmax_idx;
 		float[] Q_i = null;
 		if(i != -1) // null Q_i not accessed: Gmax=-INF if i=-1
 			Q_i = Q.get_Q(i,active_size);
-
+	
 		for(int j=0;j<active_size;j++)
 		{
 			if(y[j]==+1)
@@ -721,13 +719,13 @@ class Solver {
 						Gmax2 = G[j];
 					if (grad_diff > 0)
 					{
-						double obj_diff;
+						double obj_diff; 
 						double quad_coef = QD[i]+QD[j]-2.0*y[i]*Q_i[j];
 						if (quad_coef > 0)
 							obj_diff = -(grad_diff*grad_diff)/quad_coef;
 						else
 							obj_diff = -(grad_diff*grad_diff)/1e-12;
-
+	
 						if (obj_diff <= obj_diff_min)
 						{
 							Gmin_idx=j;
@@ -745,13 +743,13 @@ class Solver {
 						Gmax2 = -G[j];
 					if (grad_diff > 0)
 					{
-						double obj_diff;
+						double obj_diff; 
 						double quad_coef = QD[i]+QD[j]+2.0*y[i]*Q_i[j];
 						if (quad_coef > 0)
 							obj_diff = -(grad_diff*grad_diff)/quad_coef;
 						else
 							obj_diff = -(grad_diff*grad_diff)/1e-12;
-
+	
 						if (obj_diff <= obj_diff_min)
 						{
 							Gmin_idx=j;
@@ -762,7 +760,7 @@ class Solver {
 			}
 		}
 
-		if(Gmax+Gmax2 < eps || Gmin_idx == -1)
+		if(Gmax+Gmax2 < eps)
 			return 1;
 
 		working_set[0] = Gmax_idx;
@@ -771,7 +769,7 @@ class Solver {
 	}
 
 	private boolean be_shrunk(int i, double Gmax1, double Gmax2)
-	{
+	{	
 		if(is_upper_bound(i))
 		{
 			if(y[i]==+1)
@@ -783,7 +781,7 @@ class Solver {
 		{
 			if(y[i]==+1)
 				return(G[i] > Gmax2);
-			else
+			else	
 				return(G[i] > Gmax1);
 		}
 		else
@@ -801,7 +799,7 @@ class Solver {
 		{
 			if(y[i]==+1)
 			{
-				if(!is_upper_bound(i))
+				if(!is_upper_bound(i))	
 				{
 					if(-G[i] >= Gmax1)
 						Gmax1 = -G[i];
@@ -812,14 +810,14 @@ class Solver {
 						Gmax2 = G[i];
 				}
 			}
-			else
+			else		
 			{
-				if(!is_upper_bound(i))
+				if(!is_upper_bound(i))	
 				{
 					if(-G[i] >= Gmax2)
 						Gmax2 = -G[i];
 				}
-				if(!is_lower_bound(i))
+				if(!is_lower_bound(i))	
 				{
 					if(G[i] >= Gmax1)
 						Gmax1 = G[i];
@@ -827,12 +825,11 @@ class Solver {
 			}
 		}
 
-		if(unshrink == false && Gmax1 + Gmax2 <= eps*10)
+		if(unshrink == false && Gmax1 + Gmax2 <= eps*10) 
 		{
 			unshrink = true;
 			reconstruct_gradient();
 			active_size = l;
-			svm.info("*");
 		}
 
 		for(i=0;i<active_size;i++)
@@ -860,16 +857,16 @@ class Solver {
 		{
 			double yG = y[i]*G[i];
 
-			if(is_upper_bound(i))
+			if(is_lower_bound(i))
 			{
-				if(y[i] < 0)
+				if(y[i] > 0)
 					ub = Math.min(ub,yG);
 				else
 					lb = Math.max(lb,yG);
 			}
-			else if(is_lower_bound(i))
+			else if(is_upper_bound(i))
 			{
-				if(y[i] > 0)
+				if(y[i] < 0)
 					ub = Math.min(ub,yG);
 				else
 					lb = Math.max(lb,yG);
@@ -916,18 +913,18 @@ final class Solver_NU extends Solver
 		// j: minimizes the decrease of obj value
 		//    (if quadratic coefficeint <= 0, replace it with tau)
 		//    -y_j*grad(f)_j < -y_i*grad(f)_i, j in I_low(\alpha)
-
+	
 		double Gmaxp = -INF;
 		double Gmaxp2 = -INF;
 		int Gmaxp_idx = -1;
-
+	
 		double Gmaxn = -INF;
 		double Gmaxn2 = -INF;
 		int Gmaxn_idx = -1;
-
+	
 		int Gmin_idx = -1;
 		double obj_diff_min = INF;
-
+	
 		for(int t=0;t<active_size;t++)
 			if(y[t]==+1)
 			{
@@ -947,7 +944,7 @@ final class Solver_NU extends Solver
 						Gmaxn_idx = t;
 					}
 			}
-
+	
 		int ip = Gmaxp_idx;
 		int in = Gmaxn_idx;
 		float[] Q_ip = null;
@@ -956,25 +953,25 @@ final class Solver_NU extends Solver
 			Q_ip = Q.get_Q(ip,active_size);
 		if(in != -1)
 			Q_in = Q.get_Q(in,active_size);
-
+	
 		for(int j=0;j<active_size;j++)
 		{
 			if(y[j]==+1)
 			{
-				if (!is_lower_bound(j))
+				if (!is_lower_bound(j))	
 				{
 					double grad_diff=Gmaxp+G[j];
 					if (G[j] >= Gmaxp2)
 						Gmaxp2 = G[j];
 					if (grad_diff > 0)
 					{
-						double obj_diff;
+						double obj_diff; 
 						double quad_coef = QD[ip]+QD[j]-2*Q_ip[j];
 						if (quad_coef > 0)
 							obj_diff = -(grad_diff*grad_diff)/quad_coef;
 						else
 							obj_diff = -(grad_diff*grad_diff)/1e-12;
-
+	
 						if (obj_diff <= obj_diff_min)
 						{
 							Gmin_idx=j;
@@ -992,13 +989,13 @@ final class Solver_NU extends Solver
 						Gmaxn2 = -G[j];
 					if (grad_diff > 0)
 					{
-						double obj_diff;
+						double obj_diff; 
 						double quad_coef = QD[in]+QD[j]-2*Q_in[j];
 						if (quad_coef > 0)
 							obj_diff = -(grad_diff*grad_diff)/quad_coef;
 						else
 							obj_diff = -(grad_diff*grad_diff)/1e-12;
-
+	
 						if (obj_diff <= obj_diff_min)
 						{
 							Gmin_idx=j;
@@ -1009,15 +1006,15 @@ final class Solver_NU extends Solver
 			}
 		}
 
-		if(Math.max(Gmaxp+Gmaxp2,Gmaxn+Gmaxn2) < eps || Gmin_idx == -1)
+		if(Math.max(Gmaxp+Gmaxp2,Gmaxn+Gmaxn2) < eps)
 			return 1;
-
+	
 		if(y[Gmin_idx] == +1)
 			working_set[0] = Gmaxp_idx;
 		else
 			working_set[0] = Gmaxn_idx;
 		working_set[1] = Gmin_idx;
-
+	
 		return 0;
 	}
 
@@ -1027,14 +1024,14 @@ final class Solver_NU extends Solver
 		{
 			if(y[i]==+1)
 				return(-G[i] > Gmax1);
-			else
+			else	
 				return(-G[i] > Gmax4);
 		}
 		else if(is_lower_bound(i))
 		{
 			if(y[i]==+1)
 				return(G[i] > Gmax2);
-			else
+			else	
 				return(G[i] > Gmax3);
 		}
 		else
@@ -1047,7 +1044,7 @@ final class Solver_NU extends Solver
 		double Gmax2 = -INF;	// max { y_i * grad(f)_i | y_i = +1, i in I_low(\alpha) }
 		double Gmax3 = -INF;	// max { -y_i * grad(f)_i | y_i = -1, i in I_up(\alpha) }
 		double Gmax4 = -INF;	// max { y_i * grad(f)_i | y_i = -1, i in I_low(\alpha) }
-
+ 
 		// find maximal violating pair first
 		int i;
 		for(i=0;i<active_size;i++)
@@ -1063,14 +1060,14 @@ final class Solver_NU extends Solver
 			if(!is_lower_bound(i))
 			{
 				if(y[i]==+1)
-				{
+				{	
 					if(G[i] > Gmax2) Gmax2 = G[i];
 				}
 				else	if(G[i] > Gmax3) Gmax3 = G[i];
 			}
 		}
 
-		if(unshrink == false && Math.max(Gmax1+Gmax2,Gmax3+Gmax4) <= eps*10)
+		if(unshrink == false && Math.max(Gmax1+Gmax2,Gmax3+Gmax4) <= eps*10) 
 		{
 			unshrink = true;
 			reconstruct_gradient();
@@ -1092,7 +1089,7 @@ final class Solver_NU extends Solver
 				}
 			}
 	}
-
+	
 	double calculate_rho()
 	{
 		int nr_free1 = 0,nr_free2 = 0;
@@ -1104,10 +1101,10 @@ final class Solver_NU extends Solver
 		{
 			if(y[i]==+1)
 			{
-				if(is_upper_bound(i))
-					lb1 = Math.max(lb1,G[i]);
-				else if(is_lower_bound(i))
+				if(is_lower_bound(i))
 					ub1 = Math.min(ub1,G[i]);
+				else if(is_upper_bound(i))
+					lb1 = Math.max(lb1,G[i]);
 				else
 				{
 					++nr_free1;
@@ -1116,10 +1113,10 @@ final class Solver_NU extends Solver
 			}
 			else
 			{
-				if(is_upper_bound(i))
-					lb2 = Math.max(lb2,G[i]);
-				else if(is_lower_bound(i))
+				if(is_lower_bound(i))
 					ub2 = Math.min(ub2,G[i]);
+				else if(is_upper_bound(i))
+					lb2 = Math.max(lb2,G[i]);
 				else
 				{
 					++nr_free2;
@@ -1184,8 +1181,8 @@ class SVC_Q extends Kernel
 	{
 		cache.swap_index(i,j);
 		super.swap_index(i,j);
-		do {byte tmp=y[i]; y[i]=y[j]; y[j]=tmp;} while(false);
-		do {double tmp=QD[i]; QD[i]=QD[j]; QD[j]=tmp;} while(false);
+		do {byte _=y[i]; y[i]=y[j]; y[j]=_;} while(false);
+		do {double _=QD[i]; QD[i]=QD[j]; QD[j]=_;} while(false);
 	}
 }
 
@@ -1224,7 +1221,7 @@ class ONE_CLASS_Q extends Kernel
 	{
 		cache.swap_index(i,j);
 		super.swap_index(i,j);
-		do {double tmp=QD[i]; QD[i]=QD[j]; QD[j]=tmp;} while(false);
+		do {double _=QD[i]; QD[i]=QD[j]; QD[j]=_;} while(false);
 	}
 }
 
@@ -1261,9 +1258,9 @@ class SVR_Q extends Kernel
 
 	void swap_index(int i, int j)
 	{
-		do {byte tmp=sign[i]; sign[i]=sign[j]; sign[j]=tmp;} while(false);
-		do {int tmp=index[i]; index[i]=index[j]; index[j]=tmp;} while(false);
-		do {double tmp=QD[i]; QD[i]=QD[j]; QD[j]=tmp;} while(false);
+		do {byte _=sign[i]; sign[i]=sign[j]; sign[j]=_;} while(false);
+		do {int _=index[i]; index[i]=index[j]; index[j]=_;} while(false);
+		do {double _=QD[i]; QD[i]=QD[j]; QD[j]=_;} while(false);
 	}
 
 	float[] get_Q(int i, int len)
@@ -1295,7 +1292,7 @@ public class svm {
 	//
 	// construct and solve various formulations
 	//
-	public static final int LIBSVM_VERSION=330;
+	public static final int LIBSVM_VERSION=312; 
 	public static final Random rand = new Random();
 
 	private static svm_print_interface svm_print_stdout = new svm_print_interface()
@@ -1309,7 +1306,7 @@ public class svm {
 
 	private static svm_print_interface svm_print_string = svm_print_stdout;
 
-	static void info(String s)
+	static void info(String s) 
 	{
 		svm_print_string.print(s);
 	}
@@ -1473,7 +1470,7 @@ public class svm {
 		{
 			alpha2[i] = alpha2[i+l] = Math.min(sum,C);
 			sum -= alpha2[i];
-
+			
 			linear_term[i] = - prob.y[i];
 			y[i] = 1;
 
@@ -1486,7 +1483,7 @@ public class svm {
 			alpha2, C, C, param.eps, si, param.shrinking);
 
 		svm.info("epsilon = "+(-si.r)+"\n");
-
+		
 		for(i=0;i<l;i++)
 			alpha[i] = alpha2[i] - alpha2[i+l];
 	}
@@ -1497,7 +1494,7 @@ public class svm {
 	static class decision_function
 	{
 		double[] alpha;
-		double rho;
+		double rho;	
 	};
 
 	static decision_function svm_train_one(
@@ -1558,7 +1555,7 @@ public class svm {
 	}
 
 	// Platt's binary SVM Probablistic Output: an improvement from Lin et al.
-	private static void sigmoid_train(int l, double[] dec_values, double[] labels,
+	private static void sigmoid_train(int l, double[] dec_values, double[] labels, 
 				  double[] probAB)
 	{
 		double A, B;
@@ -1568,7 +1565,7 @@ public class svm {
 		for (i=0;i<l;i++)
 			if (labels[i] > 0) prior1+=1;
 			else prior0+=1;
-
+	
 		int max_iter=100;	// Maximal number of iterations
 		double min_step=1e-10;	// Minimal step taken in line search
 		double sigma=1e-12;	// For numerically strict PD of Hessian
@@ -1578,8 +1575,8 @@ public class svm {
 		double[] t= new double[l];
 		double fApB,p,q,h11,h22,h21,g1,g2,det,dA,dB,gd,stepsize;
 		double newA,newB,newf,d1,d2;
-		int iter;
-
+		int iter; 
+	
 		// Initial Point and Initial Fun Value
 		A=0.0; B=Math.log((prior0+1.0)/(prior1+1.0));
 		double fval = 0.0;
@@ -1625,7 +1622,7 @@ public class svm {
 			// Stopping Criteria
 			if (Math.abs(g1)<eps && Math.abs(g2)<eps)
 				break;
-
+			
 			// Finding Newton direction: -inv(H') * g
 			det=h11*h22-h21*h21;
 			dA=-(h22*g1 - h21 * g2) / det;
@@ -1658,14 +1655,14 @@ public class svm {
 				else
 					stepsize = stepsize / 2.0;
 			}
-
+			
 			if (stepsize < min_step)
 			{
 				svm.info("Line search fails in two-class probability estimates\n");
 				break;
 			}
 		}
-
+		
 		if (iter>=max_iter)
 			svm.info("Reaching maximal iterations in two-class probability estimates\n");
 		probAB[0]=A;probAB[1]=B;
@@ -1674,14 +1671,13 @@ public class svm {
 	private static double sigmoid_predict(double decision_value, double A, double B)
 	{
 		double fApB = decision_value*A+B;
-	// 1-p used later; avoid catastrophic cancellation
 		if (fApB >= 0)
 			return Math.exp(-fApB)/(1.0+Math.exp(-fApB));
 		else
 			return 1.0/(1+Math.exp(fApB)) ;
 	}
 
-	// Method 2 from the multiclass_prob paper by Wu, Lin, and Weng to predict probabilities
+	// Method 2 from the multiclass_prob paper by Wu, Lin, and Weng
 	private static void multiclass_probability(int k, double[][] r, double[] p)
 	{
 		int t,j;
@@ -1689,7 +1685,7 @@ public class svm {
 		double[][] Q=new double[k][k];
 		double[] Qp=new double[k];
 		double pQp, eps=0.005/k;
-
+	
 		for (t=0;t<k;t++)
 		{
 			p[t]=1.0/k;  // Valid if k = 1
@@ -1724,7 +1720,7 @@ public class svm {
 					max_error=error;
 			}
 			if (max_error<eps) break;
-
+		
 			for (t=0;t<k;t++)
 			{
 				double diff=(-Qp[t]+pQp)/Q[t][t];
@@ -1741,7 +1737,7 @@ public class svm {
 			svm.info("Exceeds max_iter in multiclass_prob\n");
 	}
 
-	// Using cross-validation decision values to get parameters for SVC probability estimates
+	// Cross-validation decision values for probability estimates
 	private static void svm_binary_svc_probability(svm_problem prob, svm_parameter param, double Cp, double Cn, double[] probAB)
 	{
 		int i;
@@ -1754,7 +1750,7 @@ public class svm {
 		for(i=0;i<prob.l;i++)
 		{
 			int j = i+rand.nextInt(prob.l-i);
-			do {int tmp=perm[i]; perm[i]=perm[j]; perm[j]=tmp;} while(false);
+			do {int _=perm[i]; perm[i]=perm[j]; perm[j]=_;} while(false);
 		}
 		for(i=0;i<nr_fold;i++)
 		{
@@ -1766,7 +1762,7 @@ public class svm {
 			subprob.l = prob.l-(end-begin);
 			subprob.x = new svm_node[subprob.l][];
 			subprob.y = new double[subprob.l];
-
+			
 			k=0;
 			for(j=0;j<begin;j++)
 			{
@@ -1786,7 +1782,7 @@ public class svm {
 					p_count++;
 				else
 					n_count++;
-
+			
 			if(p_count==0 && n_count==0)
 				for(j=begin;j<end;j++)
 					dec_values[perm[j]] = 0;
@@ -1816,82 +1812,13 @@ public class svm {
 					dec_values[perm[j]]=dec_value[0];
 					// ensure +1 -1 order; reason not using CV subroutine
 					dec_values[perm[j]] *= submodel.label[0];
-				}
+				}		
 			}
-		}
+		}		
 		sigmoid_train(prob.l,dec_values,prob.y,probAB);
 	}
 
-	// Binning method from the oneclass_prob paper by Que and Lin to predict the probability as a normal instance (i.e., not an outlier)
-	private static double predict_one_class_probability(svm_model model, double dec_value)
-	{
-		double prob_estimate = 0.0;
-		int nr_marks = 10;
-
-		if(dec_value < model.prob_density_marks[0])
-			prob_estimate = 0.0001;
-		else if(dec_value > model.prob_density_marks[nr_marks-1])
-			prob_estimate = 0.999;
-		else
-		{
-			for(int i=0;i<nr_marks;i++)
-				if(dec_value < model.prob_density_marks[i])
-				{
-					prob_estimate = (double)i/nr_marks;
-					break;
-				}
-		}
-		return prob_estimate;
-	}
-
-	// Get parameters for one-class SVM probability estimates
-	private static int svm_one_class_probability(svm_problem prob, svm_model model, double[] prob_density_marks)
-	{
-		double[] dec_values = new double[prob.l];
-		double[] pred_results = new double[prob.l];
-		int ret = 0;
-		int nr_marks = 10;
-
-		for(int i=0;i<prob.l;i++)
-		{
-			double[] dec_value = new double[1];
-			pred_results[i] = svm_predict_values(model,prob.x[i],dec_value);
-			dec_values[i] = dec_value[0];
-		}
-		Arrays.sort(dec_values);
-
-		int neg_counter=0;
-		for(int i=0;i<prob.l;i++)
-			if(dec_values[i]>=0)
-			{
-				neg_counter = i;
-				break;
-			}
-
-		int pos_counter = prob.l-neg_counter;
-		if(neg_counter<nr_marks/2 || pos_counter<nr_marks/2)
-		{
-			System.err.print("WARNING: number of positive or negative decision values <"+nr_marks/2+"; too few to do a probability estimation.\n");
-			ret = -1;
-		}
-		else
-		{
-			// Binning by density
-			double[] tmp_marks = new double[nr_marks+1];
-			int mid = nr_marks/2;
-			for(int i=0;i<mid;i++)
-				tmp_marks[i] = dec_values[i*neg_counter/mid];
-			tmp_marks[mid] = 0;
-			for(int i=mid+1;i<nr_marks+1;i++)
-				tmp_marks[i] = dec_values[neg_counter-1+(i-mid)*pos_counter/mid];
-
-			for(int i=0;i<nr_marks;i++)
-				prob_density_marks[i] = (tmp_marks[i]+tmp_marks[i+1])/2;
-		}
-		return ret;
-	}
-
-	// Return parameter of a Laplace distribution
+	// Return parameter of a Laplace distribution 
 	private static double svm_svr_probability(svm_problem prob, svm_parameter param)
 	{
 		int i;
@@ -1906,15 +1833,15 @@ public class svm {
 		{
 			ymv[i]=prob.y[i]-ymv[i];
 			mae += Math.abs(ymv[i]);
-		}
+		}		
 		mae /= prob.l;
 		double std=Math.sqrt(2*mae*mae);
 		int count=0;
 		mae=0;
 		for(i=0;i<prob.l;i++)
-			if (Math.abs(ymv[i]) > 5*std)
+			if (Math.abs(ymv[i]) > 5*std) 
 				count=count+1;
-			else
+			else 
 				mae+=Math.abs(ymv[i]);
 		mae /= (prob.l-count);
 		svm.info("Prob. model for test data: target value = predicted value + z,\nz: Laplace distribution e^(-|z|/sigma)/(2sigma),sigma="+mae+"\n");
@@ -1956,7 +1883,7 @@ public class svm {
 					label = new_data;
 					new_data = new int[max_nr_class];
 					System.arraycopy(count,0,new_data,0,count.length);
-					count = new_data;
+					count = new_data;					
 				}
 				label[nr_class] = this_label;
 				count[nr_class] = 1;
@@ -1964,24 +1891,6 @@ public class svm {
 			}
 		}
 
-		//
-		// Labels are ordered by their first occurrence in the training set.
-		// However, for two-class sets with -1/+1 labels and -1 appears first,
-		// we swap labels to ensure that internally the binary SVM has positive data corresponding to the +1 instances.
-		//
-		if (nr_class == 2 && label[0] == -1 && label[1] == +1)
-		{
-			do {int tmp=label[0]; label[0]=label[1]; label[1]=tmp;} while(false);
-			do {int tmp=count[0]; count[0]=count[1]; count[1]=tmp;} while(false);
-			for(i=0;i<l;i++)
-			{
-				if(data_label[i] == 0)
-					data_label[i] = 1;
-				else
-					data_label[i] = 0;
-			}
-		}
-
 		int[] start = new int[nr_class];
 		start[0] = 0;
 		for(i=1;i<nr_class;i++)
@@ -2018,9 +1927,16 @@ public class svm {
 			model.label = null;
 			model.nSV = null;
 			model.probA = null; model.probB = null;
-			model.prob_density_marks = null;
 			model.sv_coef = new double[1][];
 
+			if(param.probability == 1 &&
+			   (param.svm_type == svm_parameter.EPSILON_SVR ||
+			    param.svm_type == svm_parameter.NU_SVR))
+			{
+				model.probA = new double[1];
+				model.probA[0] = svm_svr_probability(prob,param);
+			}
+
 			decision_function f = svm_train_one(prob,param,0,0);
 			model.rho = new double[1];
 			model.rho[0] = f.rho;
@@ -2032,36 +1948,14 @@ public class svm {
 			model.l = nSV;
 			model.SV = new svm_node[nSV][];
 			model.sv_coef[0] = new double[nSV];
-			model.sv_indices = new int[nSV];
 			int j = 0;
 			for(i=0;i<prob.l;i++)
 				if(Math.abs(f.alpha[i]) > 0)
 				{
 					model.SV[j] = prob.x[i];
 					model.sv_coef[0][j] = f.alpha[i];
-					model.sv_indices[j] = i+1;
 					++j;
 				}
-
-			if(param.probability == 1 &&
-			   (param.svm_type == svm_parameter.EPSILON_SVR ||
-			    param.svm_type == svm_parameter.NU_SVR))
-			{
-				model.probA = new double[1];
-				model.probA[0] = svm_svr_probability(prob,param);
-			}
-			else if(param.probability == 1 && param.svm_type == svm_parameter.ONE_CLASS)
-			{
-				int nr_marks = 10;
-				double[] prob_density_marks = new double[nr_marks];
-
-				if(svm_one_class_probability(prob,model,prob_density_marks) == 0)
-				{
-					model.prob_density_marks = new double[nr_marks];
-					for(i=0;i<nr_marks;i++)
-						model.prob_density_marks[i] = prob_density_marks[i];
-				}
-			}
 		}
 		else
 		{
@@ -2070,19 +1964,19 @@ public class svm {
 			int[] tmp_nr_class = new int[1];
 			int[][] tmp_label = new int[1][];
 			int[][] tmp_start = new int[1][];
-			int[][] tmp_count = new int[1][];
+			int[][] tmp_count = new int[1][];			
 			int[] perm = new int[l];
 
 			// group training data of the same class
 			svm_group_classes(prob,tmp_nr_class,tmp_label,tmp_start,tmp_count,perm);
-			int nr_class = tmp_nr_class[0];
+			int nr_class = tmp_nr_class[0];			
 			int[] label = tmp_label[0];
 			int[] start = tmp_start[0];
 			int[] count = tmp_count[0];
-
-			if(nr_class == 1)
+ 			
+			if(nr_class == 1) 
 				svm.info("WARNING: training data in only one class. See README for details.\n");
-
+			
 			svm_node[][] x = new svm_node[l][];
 			int i;
 			for(i=0;i<l;i++)
@@ -2186,9 +2080,8 @@ public class svm {
 				model.probA=null;
 				model.probB=null;
 			}
-			model.prob_density_marks = null;	// for one-class SVM probabilistic outputs only
 
-			int total_sv = 0;
+			int nnz = 0;
 			int[] nz_count = new int[nr_class];
 			model.nSV = new int[nr_class];
 			for(i=0;i<nr_class;i++)
@@ -2198,24 +2091,19 @@ public class svm {
 					if(nonzero[start[i]+j])
 					{
 						++nSV;
-						++total_sv;
+						++nnz;
 					}
 				model.nSV[i] = nSV;
 				nz_count[i] = nSV;
 			}
 
-			svm.info("Total nSV = "+total_sv+"\n");
+			svm.info("Total nSV = "+nnz+"\n");
 
-			model.l = total_sv;
-			model.SV = new svm_node[total_sv][];
-			model.sv_indices = new int[total_sv];
+			model.l = nnz;
+			model.SV = new svm_node[nnz][];
 			p = 0;
 			for(i=0;i<l;i++)
-				if(nonzero[i])
-				{
-					model.SV[p] = x[i];
-					model.sv_indices[p++] = perm[i] + 1;
-				}
+				if(nonzero[i]) model.SV[p++] = x[i];
 
 			int[] nz_start = new int[nr_class];
 			nz_start[0] = 0;
@@ -2224,7 +2112,7 @@ public class svm {
 
 			model.sv_coef = new double[nr_class-1][];
 			for(i=0;i<nr_class-1;i++)
-				model.sv_coef[i] = new double[total_sv];
+				model.sv_coef[i] = new double[nnz];
 
 			p = 0;
 			for(i=0;i<nr_class;i++)
@@ -2253,7 +2141,7 @@ public class svm {
 		}
 		return model;
 	}
-
+	
 	// Stratified cross validation
 	public static void svm_cross_validation(svm_problem prob, svm_parameter param, int nr_fold, double[] target)
 	{
@@ -2261,7 +2149,7 @@ public class svm {
 		int[] fold_start = new int[nr_fold+1];
 		int l = prob.l;
 		int[] perm = new int[l];
-
+		
 		// stratified cv may not give leave-one-out rate
 		// Each class to l folds -> some folds may have zero elements
 		if((param.svm_type == svm_parameter.C_SVC ||
@@ -2276,7 +2164,7 @@ public class svm {
 
 			int nr_class = tmp_nr_class[0];
 			int[] start = tmp_start[0];
-			int[] count = tmp_count[0];
+			int[] count = tmp_count[0];		
 
 			// random shuffle and then data grouped by fold using the array perm
 			int[] fold_count = new int[nr_fold];
@@ -2288,7 +2176,7 @@ public class svm {
 				for(i=0;i<count[c];i++)
 				{
 					int j = i+rand.nextInt(count[c]-i);
-					do {int tmp=index[start[c]+j]; index[start[c]+j]=index[start[c]+i]; index[start[c]+i]=tmp;} while(false);
+					do {int _=index[start[c]+j]; index[start[c]+j]=index[start[c]+i]; index[start[c]+i]=_;} while(false);
 				}
 			for(i=0;i<nr_fold;i++)
 			{
@@ -2320,7 +2208,7 @@ public class svm {
 			for(i=0;i<l;i++)
 			{
 				int j = i+rand.nextInt(l-i);
-				do {int tmp=perm[i]; perm[i]=perm[j]; perm[j]=tmp;} while(false);
+				do {int _=perm[i]; perm[i]=perm[j]; perm[j]=_;} while(false);
 			}
 			for(i=0;i<=nr_fold;i++)
 				fold_start[i]=i*l/nr_fold;
@@ -2382,18 +2270,6 @@ public class svm {
 				label[i] = model.label[i];
 	}
 
-	public static void svm_get_sv_indices(svm_model model, int[] indices)
-	{
-		if (model.sv_indices != null)
-			for(int i=0;i<model.l;i++)
-				indices[i] = model.sv_indices[i];
-	}
-
-	public static int svm_get_nr_sv(svm_model model)
-	{
-		return model.l;
-	}
-
 	public static double svm_get_svr_probability(svm_model model)
 	{
 		if ((model.param.svm_type == svm_parameter.EPSILON_SVR || model.param.svm_type == svm_parameter.NU_SVR) &&
@@ -2429,7 +2305,7 @@ public class svm {
 		{
 			int nr_class = model.nr_class;
 			int l = model.l;
-
+		
 			double[] kvalue = new double[l];
 			for(i=0;i<l;i++)
 				kvalue[i] = Kernel.k_function(x,model.SV[i],model.param);
@@ -2452,7 +2328,7 @@ public class svm {
 					int sj = start[j];
 					int ci = model.nSV[i];
 					int cj = model.nSV[j];
-
+				
 					int k;
 					double[] coef1 = model.sv_coef[j-1];
 					double[] coef2 = model.sv_coef[i];
@@ -2461,7 +2337,7 @@ public class svm {
 					for(k=0;k<cj;k++)
 						sum += coef2[sj+k] * kvalue[sj+k];
 					sum -= model.rho[p];
-					dec_values[p] = sum;
+					dec_values[p] = sum;					
 
 					if(dec_values[p] > 0)
 						++vote[i];
@@ -2505,7 +2381,7 @@ public class svm {
 
 			double min_prob=1e-7;
 			double[][] pairwise_prob=new double[nr_class][nr_class];
-
+			
 			int k=0;
 			for(i=0;i<nr_class;i++)
 				for(int j=i+1;j<nr_class;j++)
@@ -2514,13 +2390,7 @@ public class svm {
 					pairwise_prob[j][i]=1-pairwise_prob[i][j];
 					k++;
 				}
-			if (nr_class == 2)
-			{
-				prob_estimates[0] = pairwise_prob[0][1];
-				prob_estimates[1] = pairwise_prob[1][0];
-			}
-			else
-				multiclass_probability(nr_class,pairwise_prob,prob_estimates);
+			multiclass_probability(nr_class,pairwise_prob,prob_estimates);
 
 			int prob_max_idx = 0;
 			for(i=1;i<nr_class;i++)
@@ -2528,15 +2398,7 @@ public class svm {
 					prob_max_idx = i;
 			return model.label[prob_max_idx];
 		}
-		else if(model.param.svm_type == svm_parameter.ONE_CLASS && model.prob_density_marks!=null)
-		{
-			double[] dec_value = new double[1];
-			double pred_result = svm_predict_values(model,x,dec_value);
-			prob_estimates[0] = predict_one_class_probability(model,dec_value[0]);
-			prob_estimates[1] = 1-prob_estimates[0];
-			return pred_result;
-		}
-		else
+		else 
 			return svm_predict(model, x);
 	}
 
@@ -2575,14 +2437,14 @@ public class svm {
 		int l = model.l;
 		fp.writeBytes("nr_class "+nr_class+"\n");
 		fp.writeBytes("total_sv "+l+"\n");
-
+	
 		{
 			fp.writeBytes("rho");
 			for(int i=0;i<nr_class*(nr_class-1)/2;i++)
 				fp.writeBytes(" "+model.rho[i]);
 			fp.writeBytes("\n");
 		}
-
+	
 		if(model.label != null)
 		{
 			fp.writeBytes("label");
@@ -2598,21 +2460,13 @@ public class svm {
 				fp.writeBytes(" "+model.probA[i]);
 			fp.writeBytes("\n");
 		}
-		if(model.probB != null)
+		if(model.probB != null) 
 		{
 			fp.writeBytes("probB");
 			for(int i=0;i<nr_class*(nr_class-1)/2;i++)
 				fp.writeBytes(" "+model.probB[i]);
 			fp.writeBytes("\n");
 		}
-		if(model.prob_density_marks != null)
-		{
-			fp.writeBytes("prob_density_marks");
-			int nr_marks = 10;
-			for(int i=0;i<nr_marks;i++)
-				fp.writeBytes(" "+model.prob_density_marks[i]);
-			fp.writeBytes("\n");
-		}
 
 		if(model.nSV != null)
 		{
@@ -2634,7 +2488,7 @@ public class svm {
 			svm_node[] p = SV[i];
 			if(param.kernel_type == svm_parameter.PRECOMPUTED)
 				fp.writeBytes("0:"+(int)(p[0].value));
-			else
+			else	
 				for(int j=0;j<p.length;j++)
 					fp.writeBytes(p[j].index+":"+p[j].value+" ");
 			fp.writeBytes("\n");
@@ -2653,132 +2507,6 @@ public class svm {
 		return Integer.parseInt(s);
 	}
 
-	private static boolean read_model_header(BufferedReader fp, svm_model model)
-	{
-		svm_parameter param = new svm_parameter();
-		model.param = param;
-		// parameters for training only won't be assigned, but arrays are assigned as null for safety
-		param.nr_weight = 0;
-		param.weight_label = null;
-		param.weight = null;
-
-		try
-		{
-			while(true)
-			{
-				String cmd = fp.readLine();
-				String arg = cmd.substring(cmd.indexOf(' ')+1);
-
-				if(cmd.startsWith("svm_type"))
-				{
-					int i;
-					for(i=0;i<svm_type_table.length;i++)
-					{
-						if(arg.indexOf(svm_type_table[i])!=-1)
-						{
-							param.svm_type=i;
-							break;
-						}
-					}
-					if(i == svm_type_table.length)
-					{
-						System.err.print("unknown svm type.\n");
-						return false;
-					}
-				}
-				else if(cmd.startsWith("kernel_type"))
-				{
-					int i;
-					for(i=0;i<kernel_type_table.length;i++)
-					{
-						if(arg.indexOf(kernel_type_table[i])!=-1)
-						{
-							param.kernel_type=i;
-							break;
-						}
-					}
-					if(i == kernel_type_table.length)
-					{
-						System.err.print("unknown kernel function.\n");
-						return false;
-					}
-				}
-				else if(cmd.startsWith("degree"))
-					param.degree = atoi(arg);
-				else if(cmd.startsWith("gamma"))
-					param.gamma = atof(arg);
-				else if(cmd.startsWith("coef0"))
-					param.coef0 = atof(arg);
-				else if(cmd.startsWith("nr_class"))
-					model.nr_class = atoi(arg);
-				else if(cmd.startsWith("total_sv"))
-					model.l = atoi(arg);
-				else if(cmd.startsWith("rho"))
-				{
-					int n = model.nr_class * (model.nr_class-1)/2;
-					model.rho = new double[n];
-					StringTokenizer st = new StringTokenizer(arg);
-					for(int i=0;i<n;i++)
-						model.rho[i] = atof(st.nextToken());
-				}
-				else if(cmd.startsWith("label"))
-				{
-					int n = model.nr_class;
-					model.label = new int[n];
-					StringTokenizer st = new StringTokenizer(arg);
-					for(int i=0;i<n;i++)
-						model.label[i] = atoi(st.nextToken());
-				}
-				else if(cmd.startsWith("probA"))
-				{
-					int n = model.nr_class*(model.nr_class-1)/2;
-					model.probA = new double[n];
-					StringTokenizer st = new StringTokenizer(arg);
-					for(int i=0;i<n;i++)
-						model.probA[i] = atof(st.nextToken());
-				}
-				else if(cmd.startsWith("probB"))
-				{
-					int n = model.nr_class*(model.nr_class-1)/2;
-					model.probB = new double[n];
-					StringTokenizer st = new StringTokenizer(arg);
-					for(int i=0;i<n;i++)
-						model.probB[i] = atof(st.nextToken());
-				}
-				else if(cmd.startsWith("prob_density_marks"))
-				{
-					int n = 10;	// nr_marks
-					model.prob_density_marks = new double[n];
-					StringTokenizer st = new StringTokenizer(arg);
-					for(int i=0;i<n;i++)
-						model.prob_density_marks[i] = atof(st.nextToken());
-				}
-				else if(cmd.startsWith("nr_sv"))
-				{
-					int n = model.nr_class;
-					model.nSV = new int[n];
-					StringTokenizer st = new StringTokenizer(arg);
-					for(int i=0;i<n;i++)
-						model.nSV[i] = atoi(st.nextToken());
-				}
-				else if(cmd.startsWith("SV"))
-				{
-					break;
-				}
-				else
-				{
-					System.err.print("unknown text in model file: ["+cmd+"]\n");
-					return false;
-				}
-			}
-		}
-		catch(Exception e)
-		{
-			return false;
-		}
-		return true;
-	}
-
 	public static svm_model svm_load_model(String model_file_name) throws IOException
 	{
 		return svm_load_model(new BufferedReader(new FileReader(model_file_name)));
@@ -2789,18 +2517,112 @@ public class svm {
 		// read parameters
 
 		svm_model model = new svm_model();
+		svm_parameter param = new svm_parameter();
+		model.param = param;
 		model.rho = null;
 		model.probA = null;
 		model.probB = null;
-		model.prob_density_marks = null;
 		model.label = null;
 		model.nSV = null;
 
-		// read header
-		if (!read_model_header(fp, model))
+		while(true)
 		{
-			System.err.print("ERROR: failed to read model\n");
-			return null;
+			String cmd = fp.readLine();
+			String arg = cmd.substring(cmd.indexOf(' ')+1);
+
+			if(cmd.startsWith("svm_type"))
+			{
+				int i;
+				for(i=0;i<svm_type_table.length;i++)
+				{
+					if(arg.indexOf(svm_type_table[i])!=-1)
+					{
+						param.svm_type=i;
+						break;
+					}
+				}
+				if(i == svm_type_table.length)
+				{
+					System.err.print("unknown svm type.\n");
+					return null;
+				}
+			}
+			else if(cmd.startsWith("kernel_type"))
+			{
+				int i;
+				for(i=0;i<kernel_type_table.length;i++)
+				{
+					if(arg.indexOf(kernel_type_table[i])!=-1)
+					{
+						param.kernel_type=i;
+						break;
+					}
+				}
+				if(i == kernel_type_table.length)
+				{
+					System.err.print("unknown kernel function.\n");
+					return null;
+				}
+			}
+			else if(cmd.startsWith("degree"))
+				param.degree = atoi(arg);
+			else if(cmd.startsWith("gamma"))
+				param.gamma = atof(arg);
+			else if(cmd.startsWith("coef0"))
+				param.coef0 = atof(arg);
+			else if(cmd.startsWith("nr_class"))
+				model.nr_class = atoi(arg);
+			else if(cmd.startsWith("total_sv"))
+				model.l = atoi(arg);
+			else if(cmd.startsWith("rho"))
+			{
+				int n = model.nr_class * (model.nr_class-1)/2;
+				model.rho = new double[n];
+				StringTokenizer st = new StringTokenizer(arg);
+				for(int i=0;i<n;i++)
+					model.rho[i] = atof(st.nextToken());
+			}
+			else if(cmd.startsWith("label"))
+			{
+				int n = model.nr_class;
+				model.label = new int[n];
+				StringTokenizer st = new StringTokenizer(arg);
+				for(int i=0;i<n;i++)
+					model.label[i] = atoi(st.nextToken());					
+			}
+			else if(cmd.startsWith("probA"))
+			{
+				int n = model.nr_class*(model.nr_class-1)/2;
+				model.probA = new double[n];
+				StringTokenizer st = new StringTokenizer(arg);
+				for(int i=0;i<n;i++)
+					model.probA[i] = atof(st.nextToken());					
+			}
+			else if(cmd.startsWith("probB"))
+			{
+				int n = model.nr_class*(model.nr_class-1)/2;
+				model.probB = new double[n];
+				StringTokenizer st = new StringTokenizer(arg);
+				for(int i=0;i<n;i++)
+					model.probB[i] = atof(st.nextToken());					
+			}
+			else if(cmd.startsWith("nr_sv"))
+			{
+				int n = model.nr_class;
+				model.nSV = new int[n];
+				StringTokenizer st = new StringTokenizer(arg);
+				for(int i=0;i<n;i++)
+					model.nSV[i] = atoi(st.nextToken());
+			}
+			else if(cmd.startsWith("SV"))
+			{
+				break;
+			}
+			else
+			{
+				System.err.print("unknown text in model file: ["+cmd+"]\n");
+				return null;
+			}
 		}
 
 		// read sv_coef and SV
@@ -2844,7 +2666,7 @@ public class svm {
 		return "unknown svm type";
 
 		// kernel_type, degree
-
+	
 		int kernel_type = param.kernel_type;
 		if(kernel_type != svm_parameter.LINEAR &&
 		   kernel_type != svm_parameter.POLY &&
@@ -2853,13 +2675,10 @@ public class svm {
 		   kernel_type != svm_parameter.PRECOMPUTED)
 			return "unknown kernel type";
 
-		if((kernel_type == svm_parameter.POLY ||
-		   kernel_type == svm_parameter.RBF ||
-		   kernel_type == svm_parameter.SIGMOID) &&
-		   param.gamma < 0)
+		if(param.gamma < 0)
 			return "gamma < 0";
 
-		if(kernel_type == svm_parameter.POLY && param.degree < 0)
+		if(param.degree < 0)
 			return "degree of polynomial kernel < 0";
 
 		// cache_size,eps,C,nu,p,shrinking
@@ -2894,8 +2713,12 @@ public class svm {
 		   param.probability != 1)
 			return "probability != 0 and probability != 1";
 
+		if(param.probability == 1 &&
+		   svm_type == svm_parameter.ONE_CLASS)
+			return "one-class SVM probability output not supported yet";
+		
 		// check whether nu-svc is feasible
-
+	
 		if(svm_type == svm_parameter.NU_SVC)
 		{
 			int l = prob.l;
@@ -2924,7 +2747,7 @@ public class svm {
 						int[] new_data = new int[max_nr_class];
 						System.arraycopy(label,0,new_data,0,label.length);
 						label = new_data;
-
+						
 						new_data = new int[max_nr_class];
 						System.arraycopy(count,0,new_data,0,count.length);
 						count = new_data;
@@ -2954,7 +2777,6 @@ public class svm {
 	{
 		if (((model.param.svm_type == svm_parameter.C_SVC || model.param.svm_type == svm_parameter.NU_SVC) &&
 		model.probA!=null && model.probB!=null) ||
-		(model.param.svm_type == svm_parameter.ONE_CLASS && model.prob_density_marks!=null) ||
 		((model.param.svm_type == svm_parameter.EPSILON_SVR || model.param.svm_type == svm_parameter.NU_SVR) &&
 		 model.probA!=null))
 			return 1;
@@ -2966,7 +2788,7 @@ public class svm {
 	{
 		if (print_func == null)
 			svm_print_string = svm_print_stdout;
-		else
+		else 
 			svm_print_string = print_func;
 	}
 }
