@@ -1,15 +1,24 @@
-/*
-	scale attributes to [lower,upper]
-	usage: scale [-l lower] [-u upper] [-y y_lower y_upper] 
-		     [-s filename] [-r filename] filename
-*/
 #include <float.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <ctype.h>
 #include <string.h>
 
-char *line;
+void exit_with_help()
+{
+	printf(
+	"Usage: svm-scale [options] data_filename\n"
+	"options:\n"
+	"-l lower : x scaling lower limit (default -1)\n"
+	"-u upper : x scaling upper limit (default +1)\n"
+	"-y y_lower y_upper : y scaling limits (default: no y scaling)\n"
+	"-s save_filename : save scaling parameters to save_filename\n"
+	"-r restore_filename : restore scaling parameters from restore_filename\n"
+	);
+	exit(1);
+}
+
+char *line = NULL;
 int max_line_len = 1024;
 double lower=-1.0,upper=1.0,y_lower,y_upper;
 int y_scaling = 0;
@@ -18,9 +27,11 @@ double *feature_min;
 double y_max = -DBL_MAX;
 double y_min = DBL_MAX;
 int max_index;
+long int num_nonzeros = 0;
+long int new_num_nonzeros = 0;
 
-#define max(x,y) ((x>y)?x:y)
-#define min(x,y) ((x<y)?x:y)
+#define max(x,y) (((x)>(y))?(x):(y))
+#define min(x,y) (((x)<(y))?(x):(y))
 
 void output_target(double value);
 void output(int index, double value);
@@ -29,7 +40,7 @@ char* readline(FILE *input);
 int main(int argc,char **argv)
 {
 	int i,index;
-	FILE *fp;
+	FILE *fp, *fp_restore = NULL;
 	char *save_filename = NULL;
 	char *restore_filename = NULL;
 
@@ -51,7 +62,7 @@ int main(int argc,char **argv)
 			case 'r': restore_filename = argv[i]; break;
 			default:
 				fprintf(stderr,"unknown option\n");
-				exit(1);
+				exit_with_help();
 		}
 	}
 
@@ -61,14 +72,15 @@ int main(int argc,char **argv)
 		exit(1);
 	}
 	
-	if(argc != i+1) 
+	if(restore_filename && save_filename)
 	{
-		fprintf(stderr,"usage: %s [-l lower] [-u upper] [-y y_lower y_upper]\n",argv[0]);
-		fprintf(stderr,"      [-s save_filename] [-r restore_filename] filename\n");
-		fprintf(stderr,"(default: lower = -1, upper = 1, no y scaling)\n");
+		fprintf(stderr,"cannot use -r and -s simultaneously\n");
 		exit(1);
 	}
 
+	if(argc != i+1) 
+		exit_with_help();
+
 	fp=fopen(argv[i],"r");
 	
 	if(fp==NULL)
@@ -92,6 +104,33 @@ int main(int argc,char **argv)
 	/* assumption: min index of attributes is 1 */
 	/* pass 1: find out max index of attributes */
 	max_index = 0;
+
+	if(restore_filename)
+	{
+		int idx, c;
+
+		fp_restore = fopen(restore_filename,"r");
+		if(fp_restore==NULL)
+		{
+			fprintf(stderr,"can't open file %s\n", restore_filename);
+			exit(1);
+		}
+
+		c = fgetc(fp_restore);
+		if(c == 'y')
+		{
+			readline(fp_restore);
+			readline(fp_restore);
+			readline(fp_restore);
+		}
+		readline(fp_restore);
+		readline(fp_restore);
+
+		while(fscanf(fp_restore,"%d %*f %*f\n",&idx) == 1)
+			max_index = max(idx,max_index);
+		rewind(fp_restore);
+	}
+
 	while(readline(fp)!=NULL)
 	{
 		char *p=line;
@@ -102,8 +141,10 @@ int main(int argc,char **argv)
 		{
 			max_index = max(max_index, index);
 			SKIP_ELEMENT
+			num_nonzeros++;
 		}		
 	}
+	rewind(fp);
 	
 	feature_max = (double *)malloc((max_index+1)* sizeof(double));
 	feature_min = (double *)malloc((max_index+1)* sizeof(double));
@@ -120,8 +161,6 @@ int main(int argc,char **argv)
 		feature_min[i]=DBL_MAX;
 	}
 
-	rewind(fp);
-
 	/* pass 2: find out min/max value */
 	while(readline(fp)!=NULL)
 	{
@@ -164,15 +203,10 @@ int main(int argc,char **argv)
 	
 	if(restore_filename)
 	{
-		FILE *fp_restore = fopen(restore_filename,"r");
+		/* fp_restore rewinded in finding max_index */
 		int idx, c;
 		double fmin, fmax;
 		
-		if(fp_restore==NULL)
-		{
-			fprintf(stderr,"can't open file %s\n", restore_filename);
-			exit(1);
-		}
 		if((c = fgetc(fp_restore)) == 'y')
 		{
 			fscanf(fp_restore, "%lf %lf\n", &y_lower, &y_upper);
@@ -225,7 +259,6 @@ int main(int argc,char **argv)
 	{
 		char *p=line;
 		int next_index=1;
-		int index;
 		double target;
 		double value;
 		
@@ -251,7 +284,16 @@ int main(int argc,char **argv)
 		printf("\n");
 	}
 
+	if (new_num_nonzeros > num_nonzeros)
+		fprintf(stderr, 
+			"WARNING: original #nonzeros %ld\n"
+			"         new      #nonzeros %ld\n"
+			"Use -l 0 if many original feature values are zeros\n",
+			num_nonzeros, new_num_nonzeros);
+
 	free(line);
+	free(feature_max);
+	free(feature_min);
 	fclose(fp);
 	return 0;
 }
@@ -267,7 +309,7 @@ char* readline(FILE *input)
 	{
 		max_line_len *= 2;
 		line = (char *) realloc(line, max_line_len);
-		len = strlen(line);
+		len = (int) strlen(line);
 		if(fgets(line+len,max_line_len-len,input) == NULL)
 			break;
 	}
@@ -304,5 +346,8 @@ void output(int index, double value)
 			(feature_max[index]-feature_min[index]);
 
 	if(value != 0)
+	{
 		printf("%d:%g ",index, value);
+		new_num_nonzeros++;
+	}
 }
