@@ -92,7 +92,7 @@ int main(int argc, char **argv)
 
 	if(error_msg)
 	{
-		fprintf(stderr,"Error: %s\n",error_msg);
+		fprintf(stderr,"ERROR: %s\n",error_msg);
 		exit(1);
 	}
 
