@@ -74,7 +74,7 @@ public:
 	// return some position p where [p,len) need to be filled
 	// (p >= len if nothing needs to be filled)
 	int get_data(const int index, Qfloat **data, int len);
-	void swap_index(int i, int j);
+	void swap_index(int i, int j);	
 private:
 	int l;
 	long int size;
@@ -308,7 +308,7 @@ double Kernel::dot(const svm_node *px, const svm_node *py)
 				++py;
 			else
 				++px;
-		}
+		}			
 	}
 	return sum;
 }
@@ -337,7 +337,7 @@ double Kernel::k_function(const svm_node *x, const svm_node *y,
 				else
 				{
 					if(x->index > y->index)
-					{
+					{	
 						sum += y->value * y->value;
 						++y;
 					}
@@ -360,7 +360,7 @@ double Kernel::k_function(const svm_node *x, const svm_node *y,
 				sum += y->value * y->value;
 				++y;
 			}
-
+			
 			return exp(-param.gamma*sum);
 		}
 		case SIGMOID:
@@ -368,7 +368,7 @@ double Kernel::k_function(const svm_node *x, const svm_node *y,
 		case PRECOMPUTED:  //x: test (validation), y: SV
 			return x[(int)(y->value)].value;
 		default:
-			return 0;  // Unreachable
+			return 0;  // Unreachable 
 	}
 }
 
@@ -444,7 +444,7 @@ protected:
 	virtual double calculate_rho();
 	virtual void do_shrinking();
 private:
-	bool be_shrunk(int i, double Gmax1, double Gmax2);
+	bool be_shrunk(int i, double Gmax1, double Gmax2);	
 };
 
 void Solver::swap_index(int i, int j)
@@ -560,7 +560,7 @@ void Solver::Solve(int l, const QMatrix& Q, const double *p_, const schar *y_,
 	int iter = 0;
 	int max_iter = max(10000000, l>INT_MAX/100 ? INT_MAX : 100*l);
 	int counter = min(l,1000)+1;
-
+	
 	while(iter < max_iter)
 	{
 		// show progress and do shrinking
@@ -585,11 +585,11 @@ void Solver::Solve(int l, const QMatrix& Q, const double *p_, const schar *y_,
 			else
 				counter = 1;	// do shrinking next iteration
 		}
-
+		
 		++iter;
 
 		// update alpha[i] and alpha[j], handle bounds carefully
-
+		
 		const Qfloat *Q_i = Q.get_Q(i,active_size);
 		const Qfloat *Q_j = Q.get_Q(j,active_size);
 
@@ -608,7 +608,7 @@ void Solver::Solve(int l, const QMatrix& Q, const double *p_, const schar *y_,
 			double diff = alpha[i] - alpha[j];
 			alpha[i] += delta;
 			alpha[j] += delta;
-
+			
 			if(diff > 0)
 			{
 				if(alpha[j] < 0)
@@ -690,7 +690,7 @@ void Solver::Solve(int l, const QMatrix& Q, const double *p_, const schar *y_,
 
 		double delta_alpha_i = alpha[i] - old_alpha_i;
 		double delta_alpha_j = alpha[j] - old_alpha_j;
-
+		
 		for(int k=0;k<active_size;k++)
 		{
 			G[k] += Q_i[k]*delta_alpha_i + Q_j[k]*delta_alpha_j;
@@ -737,7 +737,7 @@ void Solver::Solve(int l, const QMatrix& Q, const double *p_, const schar *y_,
 			active_size = l;
 			info("*");
 		}
-		fprintf(stderr,"\nWARNING: reaching max number of iterations\n");
+		info("\nWARNING: reaching max number of iterations");
 	}
 
 	// calculate rho
@@ -790,7 +790,7 @@ int Solver::select_working_set(int &out_i, int &out_j)
 	// j: minimizes the decrease of obj value
 	//    (if quadratic coefficeint <= 0, replace it with tau)
 	//    -y_j*grad(f)_j < -y_i*grad(f)_i, j in I_low(\alpha)
-
+	
 	double Gmax = -INF;
 	double Gmax2 = -INF;
 	int Gmax_idx = -1;
@@ -798,7 +798,7 @@ int Solver::select_working_set(int &out_i, int &out_j)
 	double obj_diff_min = INF;
 
 	for(int t=0;t<active_size;t++)
-		if(y[t]==+1)
+		if(y[t]==+1)	
 		{
 			if(!is_upper_bound(t))
 				if(-G[t] >= Gmax)
@@ -833,7 +833,7 @@ int Solver::select_working_set(int &out_i, int &out_j)
 					Gmax2 = G[j];
 				if (grad_diff > 0)
 				{
-					double obj_diff;
+					double obj_diff; 
 					double quad_coef = QD[i]+QD[j]-2.0*y[i]*Q_i[j];
 					if (quad_coef > 0)
 						obj_diff = -(grad_diff*grad_diff)/quad_coef;
@@ -857,7 +857,7 @@ int Solver::select_working_set(int &out_i, int &out_j)
 					Gmax2 = -G[j];
 				if (grad_diff > 0)
 				{
-					double obj_diff;
+					double obj_diff; 
 					double quad_coef = QD[i]+QD[j]+2.0*y[i]*Q_i[j];
 					if (quad_coef > 0)
 						obj_diff = -(grad_diff*grad_diff)/quad_coef;
@@ -874,7 +874,7 @@ int Solver::select_working_set(int &out_i, int &out_j)
 		}
 	}
 
-	if(Gmax+Gmax2 < eps || Gmin_idx == -1)
+	if(Gmax+Gmax2 < eps)
 		return 1;
 
 	out_i = Gmax_idx;
@@ -895,7 +895,7 @@ bool Solver::be_shrunk(int i, double Gmax1, double Gmax2)
 	{
 		if(y[i]==+1)
 			return(G[i] > Gmax2);
-		else
+		else	
 			return(G[i] > Gmax1);
 	}
 	else
@@ -911,27 +911,27 @@ void Solver::do_shrinking()
 	// find maximal violating pair first
 	for(i=0;i<active_size;i++)
 	{
-		if(y[i]==+1)
+		if(y[i]==+1)	
 		{
-			if(!is_upper_bound(i))
+			if(!is_upper_bound(i))	
 			{
 				if(-G[i] >= Gmax1)
 					Gmax1 = -G[i];
 			}
-			if(!is_lower_bound(i))
+			if(!is_lower_bound(i))	
 			{
 				if(G[i] >= Gmax2)
 					Gmax2 = G[i];
 			}
 		}
-		else
+		else	
 		{
-			if(!is_upper_bound(i))
+			if(!is_upper_bound(i))	
 			{
 				if(-G[i] >= Gmax2)
 					Gmax2 = -G[i];
 			}
-			if(!is_lower_bound(i))
+			if(!is_lower_bound(i))	
 			{
 				if(G[i] >= Gmax1)
 					Gmax1 = G[i];
@@ -939,7 +939,7 @@ void Solver::do_shrinking()
 		}
 	}
 
-	if(unshrink == false && Gmax1 + Gmax2 <= eps*10)
+	if(unshrink == false && Gmax1 + Gmax2 <= eps*10) 
 	{
 		unshrink = true;
 		reconstruct_gradient();
@@ -1006,7 +1006,7 @@ double Solver::calculate_rho()
 //
 // additional constraint: e^T \alpha = constant
 //
-class Solver_NU: public Solver
+class Solver_NU : public Solver
 {
 public:
 	Solver_NU() {}
@@ -1078,14 +1078,14 @@ int Solver_NU::select_working_set(int &out_i, int &out_j)
 	{
 		if(y[j]==+1)
 		{
-			if (!is_lower_bound(j))
+			if (!is_lower_bound(j))	
 			{
 				double grad_diff=Gmaxp+G[j];
 				if (G[j] >= Gmaxp2)
 					Gmaxp2 = G[j];
 				if (grad_diff > 0)
 				{
-					double obj_diff;
+					double obj_diff; 
 					double quad_coef = QD[ip]+QD[j]-2*Q_ip[j];
 					if (quad_coef > 0)
 						obj_diff = -(grad_diff*grad_diff)/quad_coef;
@@ -1109,7 +1109,7 @@ int Solver_NU::select_working_set(int &out_i, int &out_j)
 					Gmaxn2 = -G[j];
 				if (grad_diff > 0)
 				{
-					double obj_diff;
+					double obj_diff; 
 					double quad_coef = QD[in]+QD[j]-2*Q_in[j];
 					if (quad_coef > 0)
 						obj_diff = -(grad_diff*grad_diff)/quad_coef;
@@ -1126,7 +1126,7 @@ int Solver_NU::select_working_set(int &out_i, int &out_j)
 		}
 	}
 
-	if(max(Gmaxp+Gmaxp2,Gmaxn+Gmaxn2) < eps || Gmin_idx == -1)
+	if(max(Gmaxp+Gmaxp2,Gmaxn+Gmaxn2) < eps)
 		return 1;
 
 	if (y[Gmin_idx] == +1)
@@ -1144,14 +1144,14 @@ bool Solver_NU::be_shrunk(int i, double Gmax1, double Gmax2, double Gmax3, doubl
 	{
 		if(y[i]==+1)
 			return(-G[i] > Gmax1);
-		else
+		else	
 			return(-G[i] > Gmax4);
 	}
 	else if(is_lower_bound(i))
 	{
 		if(y[i]==+1)
 			return(G[i] > Gmax2);
-		else
+		else	
 			return(G[i] > Gmax3);
 	}
 	else
@@ -1180,14 +1180,14 @@ void Solver_NU::do_shrinking()
 		if(!is_lower_bound(i))
 		{
 			if(y[i]==+1)
-			{
+			{	
 				if(G[i] > Gmax2) Gmax2 = G[i];
 			}
 			else	if(G[i] > Gmax3) Gmax3 = G[i];
 		}
 	}
 
-	if(unshrink == false && max(Gmax1+Gmax2,Gmax3+Gmax4) <= eps*10)
+	if(unshrink == false && max(Gmax1+Gmax2,Gmax3+Gmax4) <= eps*10) 
 	{
 		unshrink = true;
 		reconstruct_gradient();
@@ -1250,12 +1250,12 @@ double Solver_NU::calculate_rho()
 		r1 = sum_free1/nr_free1;
 	else
 		r1 = (ub1+lb1)/2;
-
+	
 	if(nr_free2 > 0)
 		r2 = sum_free2/nr_free2;
 	else
 		r2 = (ub2+lb2)/2;
-
+	
 	si->r = (r1+r2)/2;
 	return (r1-r2)/2;
 }
@@ -1264,7 +1264,7 @@ double Solver_NU::calculate_rho()
 // Q matrices for various formulations
 //
 class SVC_Q: public Kernel
-{
+{ 
 public:
 	SVC_Q(const svm_problem& prob, const svm_parameter& param, const schar *y_)
 	:Kernel(prob.l, prob.x, param)
@@ -1275,7 +1275,7 @@ public:
 		for(int i=0;i<prob.l;i++)
 			QD[i] = (this->*kernel_function)(i,i);
 	}
-
+	
 	Qfloat *get_Q(int i, int len) const
 	{
 		Qfloat *data;
@@ -1324,7 +1324,7 @@ public:
 		for(int i=0;i<prob.l;i++)
 			QD[i] = (this->*kernel_function)(i,i);
 	}
-
+	
 	Qfloat *get_Q(int i, int len) const
 	{
 		Qfloat *data;
@@ -1360,7 +1360,7 @@ private:
 };
 
 class SVR_Q: public Kernel
-{
+{ 
 public:
 	SVR_Q(const svm_problem& prob, const svm_parameter& param)
 	:Kernel(prob.l, prob.x, param)
@@ -1390,7 +1390,7 @@ public:
 		swap(index[i],index[j]);
 		swap(QD[i],QD[j]);
 	}
-
+	
 	Qfloat *get_Q(int i, int len) const
 	{
 		Qfloat *data;
@@ -1641,7 +1641,7 @@ static void solve_nu_svr(
 struct decision_function
 {
 	double *alpha;
-	double rho;
+	double rho;	
 };
 
 static decision_function svm_train_one(
@@ -1703,7 +1703,7 @@ static decision_function svm_train_one(
 
 // Platt's binary SVM Probablistic Output: an improvement from Lin et al.
 static void sigmoid_train(
-	int l, const double *dec_values, const double *labels,
+	int l, const double *dec_values, const double *labels, 
 	double& A, double& B)
 {
 	double prior1=0, prior0 = 0;
@@ -1712,7 +1712,7 @@ static void sigmoid_train(
 	for (i=0;i<l;i++)
 		if (labels[i] > 0) prior1+=1;
 		else prior0+=1;
-
+	
 	int max_iter=100;	// Maximal number of iterations
 	double min_step=1e-10;	// Minimal step taken in line search
 	double sigma=1e-12;	// For numerically strict PD of Hessian
@@ -1722,8 +1722,8 @@ static void sigmoid_train(
 	double *t=Malloc(double,l);
 	double fApB,p,q,h11,h22,h21,g1,g2,det,dA,dB,gd,stepsize;
 	double newA,newB,newf,d1,d2;
-	int iter;
-
+	int iter; 
+	
 	// Initial Point and Initial Fun Value
 	A=0.0; B=log((prior0+1.0)/(prior1+1.0));
 	double fval = 0.0;
@@ -1825,7 +1825,7 @@ static double sigmoid_predict(double decision_value, double A, double B)
 		return 1.0/(1+exp(fApB)) ;
 }
 
-// Method 2 from the multiclass_prob paper by Wu, Lin, and Weng to predict probabilities
+// Method 2 from the multiclass_prob paper by Wu, Lin, and Weng
 static void multiclass_probability(int k, double **r, double *p)
 {
 	int t,j;
@@ -1833,7 +1833,7 @@ static void multiclass_probability(int k, double **r, double *p)
 	double **Q=Malloc(double *,k);
 	double *Qp=Malloc(double,k);
 	double pQp, eps=0.005/k;
-
+	
 	for (t=0;t<k;t++)
 	{
 		p[t]=1.0/k;  // Valid if k = 1
@@ -1869,7 +1869,7 @@ static void multiclass_probability(int k, double **r, double *p)
 				max_error=error;
 		}
 		if (max_error<eps) break;
-
+		
 		for (t=0;t<k;t++)
 		{
 			double diff=(-Qp[t]+pQp)/Q[t][t];
@@ -1889,7 +1889,7 @@ static void multiclass_probability(int k, double **r, double *p)
 	free(Qp);
 }
 
-// Using cross-validation decision values to get parameters for SVC probability estimates
+// Cross-validation decision values for probability estimates
 static void svm_binary_svc_probability(
 	const svm_problem *prob, const svm_parameter *param,
 	double Cp, double Cn, double& probA, double& probB)
@@ -1916,7 +1916,7 @@ static void svm_binary_svc_probability(
 		subprob.l = prob->l-(end-begin);
 		subprob.x = Malloc(struct svm_node*,subprob.l);
 		subprob.y = Malloc(double,subprob.l);
-
+			
 		k=0;
 		for(j=0;j<begin;j++)
 		{
@@ -1961,99 +1961,22 @@ static void svm_binary_svc_probability(
 			struct svm_model *submodel = svm_train(&subprob,&subparam);
 			for(j=begin;j<end;j++)
 			{
-				svm_predict_values(submodel,prob->x[perm[j]],&(dec_values[perm[j]]));
+				svm_predict_values(submodel,prob->x[perm[j]],&(dec_values[perm[j]])); 
 				// ensure +1 -1 order; reason not using CV subroutine
 				dec_values[perm[j]] *= submodel->label[0];
-			}
+			}		
 			svm_free_and_destroy_model(&submodel);
 			svm_destroy_param(&subparam);
 		}
 		free(subprob.x);
 		free(subprob.y);
-	}
+	}		
 	sigmoid_train(prob->l,dec_values,prob->y,probA,probB);
 	free(dec_values);
 	free(perm);
 }
 
-// Binning method from the oneclass_prob paper by Que and Lin to predict the probability as a normal instance (i.e., not an outlier)
-static double predict_one_class_probability(const svm_model *model, double dec_value)
-{
-	double prob_estimate = 0.0;
-	int nr_marks = 10;
-
-	if(dec_value < model->prob_density_marks[0])
-		prob_estimate = 0.001;
-	else if(dec_value > model->prob_density_marks[nr_marks-1])
-		prob_estimate = 0.999;
-	else
-	{
-		for(int i=1;i<nr_marks;i++)
-			if(dec_value < model->prob_density_marks[i])
-			{
-				prob_estimate = (double)i/nr_marks;
-				break;
-			}
-	}
-	return prob_estimate;
-}
-
-static int compare_double(const void *a, const void *b)
-{
-	if(*(double *)a > *(double *)b)
-		return 1;
-	else if(*(double *)a < *(double *)b)
-		return -1;
-	return 0;
-}
-
-// Get parameters for one-class SVM probability estimates
-static int svm_one_class_probability(const svm_problem *prob, const svm_model *model, double *prob_density_marks)
-{
-	double *dec_values = Malloc(double,prob->l);
-	double *pred_results = Malloc(double,prob->l);
-	int ret = 0;
-	int nr_marks = 10;
-
-	for(int i=0;i<prob->l;i++)
-		pred_results[i] = svm_predict_values(model,prob->x[i],&dec_values[i]);
-	qsort(dec_values,prob->l,sizeof(double),compare_double);
-
-	int neg_counter=0;
-	for(int i=0;i<prob->l;i++)
-		if(dec_values[i]>=0)
-		{
-			neg_counter = i;
-			break;
-		}
-
-	int pos_counter = prob->l-neg_counter;
-	if(neg_counter<nr_marks/2 || pos_counter<nr_marks/2)
-	{
-		fprintf(stderr,"WARNING: number of positive or negative decision values <%d; too few to do a probability estimation.\n",nr_marks/2);
-		ret = -1;
-	}
-	else
-	{
-		// Binning by density
-		double *tmp_marks = Malloc(double,nr_marks+1);
-		int mid = nr_marks/2;
-		for(int i=0;i<mid;i++)
-			tmp_marks[i] = dec_values[i*neg_counter/mid];
-		tmp_marks[mid] = 0;
-		for(int i=mid+1;i<nr_marks+1;i++)
-			tmp_marks[i] = dec_values[neg_counter-1+(i-mid)*pos_counter/mid];
-
-		for(int i=0;i<nr_marks;i++)
-			prob_density_marks[i] = (tmp_marks[i]+tmp_marks[i+1])/2;
-		free(tmp_marks);
-	}
-	free(dec_values);
-	free(pred_results);
-	return ret;
-}
-
-// Return parameter of a Laplace distribution
+// Return parameter of a Laplace distribution 
 static double svm_svr_probability(
 	const svm_problem *prob, const svm_parameter *param)
 {
@@ -2069,15 +1992,15 @@ static double svm_svr_probability(
 	{
 		ymv[i]=prob->y[i]-ymv[i];
 		mae += fabs(ymv[i]);
-	}
+	}		
 	mae /= prob->l;
 	double std=sqrt(2*mae*mae);
 	int count=0;
 	mae=0;
 	for(i=0;i<prob->l;i++)
-		if (fabs(ymv[i]) > 5*std)
+		if (fabs(ymv[i]) > 5*std) 
 			count=count+1;
-		else
+		else 
 			mae+=fabs(ymv[i]);
 	mae /= (prob->l-count);
 	info("Prob. model for test data: target value = predicted value + z,\nz: Laplace distribution e^(-|z|/sigma)/(2sigma),sigma= %g\n",mae);
@@ -2095,7 +2018,7 @@ static void svm_group_classes(const svm_problem *prob, int *nr_class_ret, int **
 	int nr_class = 0;
 	int *label = Malloc(int,max_nr_class);
 	int *count = Malloc(int,max_nr_class);
-	int *data_label = Malloc(int,l);
+	int *data_label = Malloc(int,l);	
 	int i;
 
 	for(i=0;i<l;i++)
@@ -2125,24 +2048,6 @@ static void svm_group_classes(const svm_problem *prob, int *nr_class_ret, int **
 		}
 	}
 
-	//
-	// Labels are ordered by their first occurrence in the training set.
-	// However, for two-class sets with -1/+1 labels and -1 appears first,
-	// we swap labels to ensure that internally the binary SVM has positive data corresponding to the +1 instances.
-	//
-	if (nr_class == 2 && label[0] == -1 && label[1] == 1)
-	{
-		swap(label[0],label[1]);
-		swap(count[0],count[1]);
-		for(i=0;i<l;i++)
-		{
-			if(data_label[i] == 0)
-				data_label[i] = 1;
-			else
-				data_label[i] = 0;
-		}
-	}
-
 	int *start = Malloc(int,nr_class);
 	start[0] = 0;
 	for(i=1;i<nr_class;i++)
@@ -2181,9 +2086,16 @@ svm_model *svm_train(const svm_problem *prob, const svm_parameter *param)
 		model->label = NULL;
 		model->nSV = NULL;
 		model->probA = NULL; model->probB = NULL;
-		model->prob_density_marks = NULL;
 		model->sv_coef = Malloc(double *,1);
 
+		if(param->probability && 
+		   (param->svm_type == EPSILON_SVR ||
+		    param->svm_type == NU_SVR))
+		{
+			model->probA = Malloc(double,1);
+			model->probA[0] = svm_svr_probability(prob,param);
+		}
+
 		decision_function f = svm_train_one(prob,param,0,0);
 		model->rho = Malloc(double,1);
 		model->rho[0] = f.rho;
@@ -2195,36 +2107,14 @@ svm_model *svm_train(const svm_problem *prob, const svm_parameter *param)
 		model->l = nSV;
 		model->SV = Malloc(svm_node *,nSV);
 		model->sv_coef[0] = Malloc(double,nSV);
-		model->sv_indices = Malloc(int,nSV);
 		int j = 0;
 		for(i=0;i<prob->l;i++)
 			if(fabs(f.alpha[i]) > 0)
 			{
 				model->SV[j] = prob->x[i];
 				model->sv_coef[0][j] = f.alpha[i];
-				model->sv_indices[j] = i+1;
 				++j;
-			}
-
-		if(param->probability &&
-		   (param->svm_type == EPSILON_SVR ||
-		    param->svm_type == NU_SVR))
-		{
-			model->probA = Malloc(double,1);
-			model->probA[0] = svm_svr_probability(prob,param);
-		}
-		else if(param->probability && param->svm_type == ONE_CLASS)
-		{
-			int nr_marks = 10;
-			double *prob_density_marks = Malloc(double,nr_marks);
-
-			if(svm_one_class_probability(prob,model,prob_density_marks) == 0)
-			{
-				model->prob_density_marks = Malloc(double,nr_marks);
-				for(i=0;i<nr_marks;i++)
-					model->prob_density_marks[i] = prob_density_marks[i];
-			}
-		}
+			}		
 
 		free(f.alpha);
 	}
@@ -2240,9 +2130,9 @@ svm_model *svm_train(const svm_problem *prob, const svm_parameter *param)
 
 		// group training data of the same class
 		svm_group_classes(prob,&nr_class,&label,&start,&count,perm);
-		if(nr_class == 1)
+		if(nr_class == 1) 
 			info("WARNING: training data in only one class. See README for details.\n");
-
+		
 		svm_node **x = Malloc(svm_node *,l);
 		int i;
 		for(i=0;i<l;i++)
@@ -2254,7 +2144,7 @@ svm_model *svm_train(const svm_problem *prob, const svm_parameter *param)
 		for(i=0;i<nr_class;i++)
 			weighted_C[i] = param->C;
 		for(i=0;i<param->nr_weight;i++)
-		{
+		{	
 			int j;
 			for(j=0;j<nr_class;j++)
 				if(param->weight_label[i] == label[j])
@@ -2266,7 +2156,7 @@ svm_model *svm_train(const svm_problem *prob, const svm_parameter *param)
 		}
 
 		// train k*(k-1)/2 models
-
+		
 		bool *nonzero = Malloc(bool,l);
 		for(i=0;i<l;i++)
 			nonzero[i] = false;
@@ -2319,11 +2209,11 @@ svm_model *svm_train(const svm_problem *prob, const svm_parameter *param)
 		// build output
 
 		model->nr_class = nr_class;
-
+		
 		model->label = Malloc(int,nr_class);
 		for(i=0;i<nr_class;i++)
 			model->label[i] = label[i];
-
+		
 		model->rho = Malloc(double,nr_class*(nr_class-1)/2);
 		for(i=0;i<nr_class*(nr_class-1)/2;i++)
 			model->rho[i] = f[i].rho;
@@ -2343,7 +2233,6 @@ svm_model *svm_train(const svm_problem *prob, const svm_parameter *param)
 			model->probA=NULL;
 			model->probB=NULL;
 		}
-		model->prob_density_marks=NULL;	// for one-class SVM probabilistic outputs only
 
 		int total_sv = 0;
 		int *nz_count = Malloc(int,nr_class);
@@ -2353,26 +2242,21 @@ svm_model *svm_train(const svm_problem *prob, const svm_parameter *param)
 			int nSV = 0;
 			for(int j=0;j<count[i];j++)
 				if(nonzero[start[i]+j])
-				{
+				{	
 					++nSV;
 					++total_sv;
 				}
 			model->nSV[i] = nSV;
 			nz_count[i] = nSV;
 		}
-
+		
 		info("Total nSV = %d\n",total_sv);
 
 		model->l = total_sv;
 		model->SV = Malloc(svm_node *,total_sv);
-		model->sv_indices = Malloc(int,total_sv);
 		p = 0;
 		for(i=0;i<l;i++)
-			if(nonzero[i])
-			{
-				model->SV[p] = x[i];
-				model->sv_indices[p++] = perm[i] + 1;
-			}
+			if(nonzero[i]) model->SV[p++] = x[i];
 
 		int *nz_start = Malloc(int,nr_class);
 		nz_start[0] = 0;
@@ -2395,7 +2279,7 @@ svm_model *svm_train(const svm_problem *prob, const svm_parameter *param)
 				int sj = start[j];
 				int ci = count[i];
 				int cj = count[j];
-
+				
 				int q = nz_start[i];
 				int k;
 				for(k=0;k<ci;k++)
@@ -2407,7 +2291,7 @@ svm_model *svm_train(const svm_problem *prob, const svm_parameter *param)
 						model->sv_coef[i][q++] = f[p].alpha[ci+k];
 				++p;
 			}
-
+		
 		free(label);
 		free(probA);
 		free(probB);
@@ -2430,16 +2314,11 @@ svm_model *svm_train(const svm_problem *prob, const svm_parameter *param)
 void svm_cross_validation(const svm_problem *prob, const svm_parameter *param, int nr_fold, double *target)
 {
 	int i;
-	int *fold_start;
+	int *fold_start = Malloc(int,nr_fold+1);
 	int l = prob->l;
 	int *perm = Malloc(int,l);
 	int nr_class;
-	if (nr_fold > l)
-	{
-		nr_fold = l;
-		fprintf(stderr,"WARNING: # folds > # data. Will use # folds = # data instead (i.e., leave-one-out cross validation)\n");
-	}
-	fold_start = Malloc(int,nr_fold+1);
+
 	// stratified cv may not give leave-one-out rate
 	// Each class to l folds -> some folds may have zero elements
 	if((param->svm_type == C_SVC ||
@@ -2456,7 +2335,7 @@ void svm_cross_validation(const svm_problem *prob, const svm_parameter *param, i
 		int *index = Malloc(int,l);
 		for(i=0;i<l;i++)
 			index[i]=perm[i];
-		for (c=0; c<nr_class; c++)
+		for (c=0; c<nr_class; c++) 
 			for(i=0;i<count[c];i++)
 			{
 				int j = i+rand()%(count[c]-i);
@@ -2485,9 +2364,9 @@ void svm_cross_validation(const svm_problem *prob, const svm_parameter *param, i
 		fold_start[0]=0;
 		for (i=1;i<=nr_fold;i++)
 			fold_start[i] = fold_start[i-1]+fold_count[i-1];
-		free(start);
+		free(start);	
 		free(label);
-		free(count);
+		free(count);	
 		free(index);
 		free(fold_count);
 	}
@@ -2513,7 +2392,7 @@ void svm_cross_validation(const svm_problem *prob, const svm_parameter *param, i
 		subprob.l = l-(end-begin);
 		subprob.x = Malloc(struct svm_node*,subprob.l);
 		subprob.y = Malloc(double,subprob.l);
-
+			
 		k=0;
 		for(j=0;j<begin;j++)
 		{
@@ -2528,13 +2407,13 @@ void svm_cross_validation(const svm_problem *prob, const svm_parameter *param, i
 			++k;
 		}
 		struct svm_model *submodel = svm_train(&subprob,param);
-		if(param->probability &&
+		if(param->probability && 
 		   (param->svm_type == C_SVC || param->svm_type == NU_SVC))
 		{
 			double *prob_estimates=Malloc(double,svm_get_nr_class(submodel));
 			for(j=begin;j<end;j++)
 				target[perm[j]] = svm_predict_probability(submodel,prob->x[perm[j]],prob_estimates);
-			free(prob_estimates);
+			free(prob_estimates);			
 		}
 		else
 			for(j=begin;j<end;j++)
@@ -2542,9 +2421,9 @@ void svm_cross_validation(const svm_problem *prob, const svm_parameter *param, i
 		svm_free_and_destroy_model(&submodel);
 		free(subprob.x);
 		free(subprob.y);
-	}
+	}		
 	free(fold_start);
-	free(perm);
+	free(perm);	
 }
 
 
@@ -2565,18 +2444,6 @@ void svm_get_labels(const svm_model *model, int* label)
 			label[i] = model->label[i];
 }
 
-void svm_get_sv_indices(const svm_model *model, int* indices)
-{
-	if (model->sv_indices != NULL)
-		for(int i=0;i<model->l;i++)
-			indices[i] = model->sv_indices[i];
-}
-
-int svm_get_nr_sv(const svm_model *model)
-{
-	return model->l;
-}
-
 double svm_get_svr_probability(const svm_model *model)
 {
 	if ((model->param.svm_type == EPSILON_SVR || model->param.svm_type == NU_SVR) &&
@@ -2612,7 +2479,7 @@ double svm_predict_values(const svm_model *model, const svm_node *x, double* dec
 	{
 		int nr_class = model->nr_class;
 		int l = model->l;
-
+		
 		double *kvalue = Malloc(double,l);
 		for(i=0;i<l;i++)
 			kvalue[i] = Kernel::k_function(x,model->SV[i],model->param);
@@ -2635,7 +2502,7 @@ double svm_predict_values(const svm_model *model, const svm_node *x, double* dec
 				int sj = start[j];
 				int ci = model->nSV[i];
 				int cj = model->nSV[j];
-
+				
 				int k;
 				double *coef1 = model->sv_coef[j-1];
 				double *coef2 = model->sv_coef[i];
@@ -2673,7 +2540,7 @@ double svm_predict(const svm_model *model, const svm_node *x)
 	   model->param.svm_type == EPSILON_SVR ||
 	   model->param.svm_type == NU_SVR)
 		dec_values = Malloc(double, 1);
-	else
+	else 
 		dec_values = Malloc(double, nr_class*(nr_class-1)/2);
 	double pred_result = svm_predict_values(model, x, dec_values);
 	free(dec_values);
@@ -2703,13 +2570,7 @@ double svm_predict_probability(
 				pairwise_prob[j][i]=1-pairwise_prob[i][j];
 				k++;
 			}
-		if (nr_class == 2)
-		{
-			prob_estimates[0] = pairwise_prob[0][1];
-			prob_estimates[1] = pairwise_prob[1][0];
-		}
-		else
-			multiclass_probability(nr_class,pairwise_prob,prob_estimates);
+		multiclass_probability(nr_class,pairwise_prob,prob_estimates);
 
 		int prob_max_idx = 0;
 		for(i=1;i<nr_class;i++)
@@ -2718,18 +2579,10 @@ double svm_predict_probability(
 		for(i=0;i<nr_class;i++)
 			free(pairwise_prob[i]);
 		free(dec_values);
-		free(pairwise_prob);
+		free(pairwise_prob);	     
 		return model->label[prob_max_idx];
 	}
-	else if(model->param.svm_type == ONE_CLASS && model->prob_density_marks!=NULL)
-	{
-		double dec_value;
-		double pred_result = svm_predict_values(model,x,&dec_value);
-		prob_estimates[0] = predict_one_class_probability(model,dec_value);
-		prob_estimates[1] = 1-prob_estimates[0];
-		return pred_result;
-	}
-	else
+	else 
 		return svm_predict(model, x);
 }
 
@@ -2748,10 +2601,7 @@ int svm_save_model(const char *model_file_name, const svm_model *model)
 	FILE *fp = fopen(model_file_name,"w");
 	if(fp==NULL) return -1;
 
-	char *old_locale = setlocale(LC_ALL, NULL);
-	if (old_locale) {
-		old_locale = strdup(old_locale);
-	}
+	char *old_locale = strdup(setlocale(LC_ALL, NULL));
 	setlocale(LC_ALL, "C");
 
 	const svm_parameter& param = model->param;
@@ -2763,23 +2613,23 @@ int svm_save_model(const char *model_file_name, const svm_model *model)
 		fprintf(fp,"degree %d\n", param.degree);
 
 	if(param.kernel_type == POLY || param.kernel_type == RBF || param.kernel_type == SIGMOID)
-		fprintf(fp,"gamma %.17g\n", param.gamma);
+		fprintf(fp,"gamma %g\n", param.gamma);
 
 	if(param.kernel_type == POLY || param.kernel_type == SIGMOID)
-		fprintf(fp,"coef0 %.17g\n", param.coef0);
+		fprintf(fp,"coef0 %g\n", param.coef0);
 
 	int nr_class = model->nr_class;
 	int l = model->l;
 	fprintf(fp, "nr_class %d\n", nr_class);
 	fprintf(fp, "total_sv %d\n",l);
-
+	
 	{
 		fprintf(fp, "rho");
 		for(int i=0;i<nr_class*(nr_class-1)/2;i++)
-			fprintf(fp," %.17g",model->rho[i]);
+			fprintf(fp," %g",model->rho[i]);
 		fprintf(fp, "\n");
 	}
-
+	
 	if(model->label)
 	{
 		fprintf(fp, "label");
@@ -2792,22 +2642,14 @@ int svm_save_model(const char *model_file_name, const svm_model *model)
 	{
 		fprintf(fp, "probA");
 		for(int i=0;i<nr_class*(nr_class-1)/2;i++)
-			fprintf(fp," %.17g",model->probA[i]);
+			fprintf(fp," %g",model->probA[i]);
 		fprintf(fp, "\n");
 	}
 	if(model->probB)
 	{
 		fprintf(fp, "probB");
 		for(int i=0;i<nr_class*(nr_class-1)/2;i++)
-			fprintf(fp," %.17g",model->probB[i]);
-		fprintf(fp, "\n");
-	}
-	if(model->prob_density_marks)
-	{
-		fprintf(fp, "prob_density_marks");
-		int nr_marks=10;
-		for(int i=0;i<nr_marks;i++)
-			fprintf(fp," %.17g",model->prob_density_marks[i]);
+			fprintf(fp," %g",model->probB[i]);
 		fprintf(fp, "\n");
 	}
 
@@ -2826,7 +2668,7 @@ int svm_save_model(const char *model_file_name, const svm_model *model)
 	for(int i=0;i<l;i++)
 	{
 		for(int j=0;j<nr_class-1;j++)
-			fprintf(fp, "%.17g ",sv_coef[j][i]);
+			fprintf(fp, "%.16g ",sv_coef[j][i]);
 
 		const svm_node *p = SV[i];
 
@@ -2869,30 +2711,32 @@ static char* readline(FILE *input)
 	return line;
 }
 
-//
-// FSCANF helps to handle fscanf failures.
-// Its do-while block avoids the ambiguity when
-// if (...)
-//    FSCANF();
-// is used
-//
-#define FSCANF(_stream, _format, _var) do{ if (fscanf(_stream, _format, _var) != 1) return false; }while(0)
-bool read_model_header(FILE *fp, svm_model* model)
+svm_model *svm_load_model(const char *model_file_name)
 {
+	FILE *fp = fopen(model_file_name,"rb");
+	if(fp==NULL) return NULL;
+
+	char *old_locale = strdup(setlocale(LC_ALL, NULL));
+	setlocale(LC_ALL, "C");
+
+	// read parameters
+
+	svm_model *model = Malloc(svm_model,1);
 	svm_parameter& param = model->param;
-	// parameters for training only won't be assigned, but arrays are assigned as NULL for safety
-	param.nr_weight = 0;
-	param.weight_label = NULL;
-	param.weight = NULL;
+	model->rho = NULL;
+	model->probA = NULL;
+	model->probB = NULL;
+	model->label = NULL;
+	model->nSV = NULL;
 
 	char cmd[81];
 	while(1)
 	{
-		FSCANF(fp,"%80s",cmd);
+		fscanf(fp,"%80s",cmd);
 
 		if(strcmp(cmd,"svm_type")==0)
 		{
-			FSCANF(fp,"%80s",cmd);
+			fscanf(fp,"%80s",cmd);
 			int i;
 			for(i=0;svm_type_table[i];i++)
 			{
@@ -2905,12 +2749,19 @@ bool read_model_header(FILE *fp, svm_model* model)
 			if(svm_type_table[i] == NULL)
 			{
 				fprintf(stderr,"unknown svm type.\n");
-				return false;
+				
+				setlocale(LC_ALL, old_locale);
+				free(old_locale);
+				free(model->rho);
+				free(model->label);
+				free(model->nSV);
+				free(model);
+				return NULL;
 			}
 		}
 		else if(strcmp(cmd,"kernel_type")==0)
-		{
-			FSCANF(fp,"%80s",cmd);
+		{		
+			fscanf(fp,"%80s",cmd);
 			int i;
 			for(i=0;kernel_type_table[i];i++)
 			{
@@ -2923,116 +2774,84 @@ bool read_model_header(FILE *fp, svm_model* model)
 			if(kernel_type_table[i] == NULL)
 			{
 				fprintf(stderr,"unknown kernel function.\n");
-				return false;
+				
+				setlocale(LC_ALL, old_locale);
+				free(old_locale);
+				free(model->rho);
+				free(model->label);
+				free(model->nSV);
+				free(model);
+				return NULL;
 			}
 		}
 		else if(strcmp(cmd,"degree")==0)
-			FSCANF(fp,"%d",&param.degree);
+			fscanf(fp,"%d",&param.degree);
 		else if(strcmp(cmd,"gamma")==0)
-			FSCANF(fp,"%lf",&param.gamma);
+			fscanf(fp,"%lf",&param.gamma);
 		else if(strcmp(cmd,"coef0")==0)
-			FSCANF(fp,"%lf",&param.coef0);
+			fscanf(fp,"%lf",&param.coef0);
 		else if(strcmp(cmd,"nr_class")==0)
-			FSCANF(fp,"%d",&model->nr_class);
+			fscanf(fp,"%d",&model->nr_class);
 		else if(strcmp(cmd,"total_sv")==0)
-			FSCANF(fp,"%d",&model->l);
+			fscanf(fp,"%d",&model->l);
 		else if(strcmp(cmd,"rho")==0)
 		{
 			int n = model->nr_class * (model->nr_class-1)/2;
 			model->rho = Malloc(double,n);
 			for(int i=0;i<n;i++)
-				FSCANF(fp,"%lf",&model->rho[i]);
+				fscanf(fp,"%lf",&model->rho[i]);
 		}
 		else if(strcmp(cmd,"label")==0)
 		{
 			int n = model->nr_class;
 			model->label = Malloc(int,n);
 			for(int i=0;i<n;i++)
-				FSCANF(fp,"%d",&model->label[i]);
+				fscanf(fp,"%d",&model->label[i]);
 		}
 		else if(strcmp(cmd,"probA")==0)
 		{
 			int n = model->nr_class * (model->nr_class-1)/2;
 			model->probA = Malloc(double,n);
 			for(int i=0;i<n;i++)
-				FSCANF(fp,"%lf",&model->probA[i]);
+				fscanf(fp,"%lf",&model->probA[i]);
 		}
 		else if(strcmp(cmd,"probB")==0)
 		{
 			int n = model->nr_class * (model->nr_class-1)/2;
 			model->probB = Malloc(double,n);
 			for(int i=0;i<n;i++)
-				FSCANF(fp,"%lf",&model->probB[i]);
-		}
-		else if(strcmp(cmd,"prob_density_marks")==0)
-		{
-			int n = 10;	// nr_marks
-			model->prob_density_marks = Malloc(double,n);
-			for(int i=0;i<n;i++)
-				FSCANF(fp,"%lf",&model->prob_density_marks[i]);
+				fscanf(fp,"%lf",&model->probB[i]);
 		}
 		else if(strcmp(cmd,"nr_sv")==0)
 		{
 			int n = model->nr_class;
 			model->nSV = Malloc(int,n);
 			for(int i=0;i<n;i++)
-				FSCANF(fp,"%d",&model->nSV[i]);
+				fscanf(fp,"%d",&model->nSV[i]);
 		}
 		else if(strcmp(cmd,"SV")==0)
 		{
 			while(1)
 			{
 				int c = getc(fp);
-				if(c==EOF || c=='\n') break;
+				if(c==EOF || c=='\n') break;	
 			}
 			break;
 		}
 		else
 		{
 			fprintf(stderr,"unknown text in model file: [%s]\n",cmd);
-			return false;
+			
+			setlocale(LC_ALL, old_locale);
+			free(old_locale);
+			free(model->rho);
+			free(model->label);
+			free(model->nSV);
+			free(model);
+			return NULL;
 		}
 	}
 
-	return true;
-
-}
-
-svm_model *svm_load_model(const char *model_file_name)
-{
-	FILE *fp = fopen(model_file_name,"rb");
-	if(fp==NULL) return NULL;
-
-	char *old_locale = setlocale(LC_ALL, NULL);
-	if (old_locale) {
-		old_locale = strdup(old_locale);
-	}
-	setlocale(LC_ALL, "C");
-
-	// read parameters
-
-	svm_model *model = Malloc(svm_model,1);
-	model->rho = NULL;
-	model->probA = NULL;
-	model->probB = NULL;
-	model->prob_density_marks = NULL;
-	model->sv_indices = NULL;
-	model->label = NULL;
-	model->nSV = NULL;
-
-	// read header
-	if (!read_model_header(fp, model))
-	{
-		fprintf(stderr, "ERROR: fscanf failed to read model\n");
-		setlocale(LC_ALL, old_locale);
-		free(old_locale);
-		free(model->rho);
-		free(model->label);
-		free(model->nSV);
-		free(model);
-		return NULL;
-	}
-
 	// read sv_coef and SV
 
 	int elements = 0;
@@ -3127,19 +2946,13 @@ void svm_free_model_content(svm_model* model_ptr)
 	model_ptr->rho = NULL;
 
 	free(model_ptr->label);
-	model_ptr->label = NULL;
+	model_ptr->label= NULL;
 
 	free(model_ptr->probA);
 	model_ptr->probA = NULL;
 
 	free(model_ptr->probB);
-	model_ptr->probB = NULL;
-
-	free(model_ptr->prob_density_marks);
-	model_ptr->prob_density_marks = NULL;
-
-	free(model_ptr->sv_indices);
-	model_ptr->sv_indices = NULL;
+	model_ptr->probB= NULL;
 
 	free(model_ptr->nSV);
 	model_ptr->nSV = NULL;
@@ -3172,9 +2985,9 @@ const char *svm_check_parameter(const svm_problem *prob, const svm_parameter *pa
 	   svm_type != EPSILON_SVR &&
 	   svm_type != NU_SVR)
 		return "unknown svm type";
-
+	
 	// kernel_type, degree
-
+	
 	int kernel_type = param->kernel_type;
 	if(kernel_type != LINEAR &&
 	   kernel_type != POLY &&
@@ -3183,11 +2996,10 @@ const char *svm_check_parameter(const svm_problem *prob, const svm_parameter *pa
 	   kernel_type != PRECOMPUTED)
 		return "unknown kernel type";
 
-	if((kernel_type == POLY || kernel_type == RBF || kernel_type == SIGMOID) &&
-	   param->gamma < 0)
+	if(param->gamma < 0)
 		return "gamma < 0";
 
-	if(kernel_type == POLY && param->degree < 0)
+	if(param->degree < 0)
 		return "degree of polynomial kernel < 0";
 
 	// cache_size,eps,C,nu,p,shrinking
@@ -3222,9 +3034,13 @@ const char *svm_check_parameter(const svm_problem *prob, const svm_parameter *pa
 	   param->probability != 1)
 		return "probability != 0 and probability != 1";
 
+	if(param->probability == 1 &&
+	   svm_type == ONE_CLASS)
+		return "one-class SVM probability output not supported yet";
+
 
 	// check whether nu-svc is feasible
-
+	
 	if(svm_type == NU_SVC)
 	{
 		int l = prob->l;
@@ -3257,7 +3073,7 @@ const char *svm_check_parameter(const svm_problem *prob, const svm_parameter *pa
 				++nr_class;
 			}
 		}
-
+	
 		for(i=0;i<nr_class;i++)
 		{
 			int n1 = count[i];
@@ -3281,10 +3097,8 @@ const char *svm_check_parameter(const svm_problem *prob, const svm_parameter *pa
 
 int svm_check_probability_model(const svm_model *model)
 {
-	return
-		((model->param.svm_type == C_SVC || model->param.svm_type == NU_SVC) &&
-		 model->probA!=NULL && model->probB!=NULL) ||
-		(model->param.svm_type == ONE_CLASS && model->prob_density_marks!=NULL) ||
+	return ((model->param.svm_type == C_SVC || model->param.svm_type == NU_SVC) &&
+		model->probA!=NULL && model->probB!=NULL) ||
 		((model->param.svm_type == EPSILON_SVR || model->param.svm_type == NU_SVR) &&
 		 model->probA!=NULL);
 }
